/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// CNR base class and subclass implementations


#include "CNR.h"
#include "Fishery.h"
#include "HvMort.h"
#include "log.h"

double CNR::sublegalMort(const HvMort* hv) const
{
    if (hv->shakers() > 0.0)
        return (hv->shakers() * sublegal_ratio(hv->fishery(), hv->region()));
    else 
        return 0.0;
}

double CNR::legalMort(const HvMort* hv) const
{
    if (hv->legalCatch() > 0.0)
        return (hv->legalCatch() * legal_ratio(hv->fishery(), hv->region())); 
    else
        return 0.0;
}


// Harvest Ratio (RT) methods
//   these use relative harvest rates from the fishery, as computed by
//   the ceiling management routines.


double CNR_HarvestRatio::sublegal_ratio(const Fishery& f, const Region& r) const
{
    double rel_hr = f.relHr(r);
    
    if (rel_hr == 0) {
        LogMsg(L_Warning, "relative harvest rate = 0.0 in fishery %s,"
            "\n\tCNR computations will be inaccurate.",
            f.get_name());
        return 0;
    }
    
    return (sublegal_sel * (1.0 - rel_hr) / rel_hr);
}

double CNR_HarvestRatio::legal_ratio(const Fishery& f, const Region& r) const
{
    double rel_hr = f.relHr(r);
    
    if (rel_hr == 0) {
        LogMsg(L_Warning, "relative harvest rate = 0.0 in fishery %s,"
            "\n\tCNR computations will be inaccurate.",
            f.get_name());
        return 0;
    }

    return (f.subLegalCnrMortRate(r) * legal_sel * (1.0 - rel_hr) / rel_hr);
}

/*
double CNR_WildAssGuess::sublegal_ratio(const Fishery&, const Region&) const
{
    return (sublegal_sel*((1.0-cnr_season_fraction)/cnr_season_fraction));
}

double CNR_WildAssGuess::legal_ratio(const Fishery&, const Region&) const
{
    return    (legal_sel*((1.0-cnr_season_fraction)/cnr_season_fraction));
}
*/

// Season Length method

double CNR_SeasonLength::sublegal_ratio(const Fishery&, const Region& ) const
{
    return (sublegal_sel * (cnr_season_len / legal_season_len));
}

double CNR_SeasonLength::legal_ratio(const Fishery& f, const Region& r) const
{
    return (f.subLegalCnrMortRate(r) * legal_sel * (cnr_season_len / legal_season_len));
}


// Reported Encounter method
//   this uses the encounter rate as computed in the shaker calculations

double CNR_ReportedEncounter::sublegal_ratio(const Fishery& f, const Region&) const
{
    return (f.encounterRate() ?
	    rpt_sublegal_enc / (f.encounterRate() * rpt_catch) : 0.0);
}

double CNR_ReportedEncounter::legal_ratio(const Fishery& f, const Region& r) const
{
    return (f.subLegalCnrMortRate(r) * rpt_legal_enc / rpt_catch);
}



