/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// CNR (Chinook Non-Retention) base class and subclasses.

#ifndef CNR_H
#define CNR_H

//#include <cnrVisitor.h>
//class cnrVisitor;
class Fishery;
class IOCrud;
class Region;
class HvMort;

// CNR abstract base class

class CNR {
  public:
    CNR(double lsel, double slsel) : legal_sel(lsel), sublegal_sel(slsel) {}
    virtual ~CNR() {}
    
    virtual double sublegalMort(const HvMort*) const;
    virtual double legalMort(const HvMort*) const;

  protected:
    virtual double sublegal_ratio(const Fishery& f, const Region& r) const = 0;
    virtual double legal_ratio(const Fishery& f, const Region& r) const = 0;

    double legal_sel;			// legal and sublegal selectivity
    double sublegal_sel;		
};

// CNR subclasses; one for each computation method

// Harvest Ratio (RT) method
class CNR_HarvestRatio : public CNR {
  public:
    CNR_HarvestRatio(double legal, double sublegal) : CNR(legal, sublegal)  {}

  protected:
    virtual double sublegal_ratio(const Fishery& f, const Region& r) const;
    virtual double legal_ratio(const Fishery& f, const Region& r) const;
};    

// Wild Ass Guess method
/*

  When using this method, the operator guesses what the
  future encounter rates will be, rather than computing them from model
  outputs (as with the RT method) or observed data (as with the season length
  or encounter rate methods). 

 */

//class CNR_WildAssGuess : public CNR {
//    friend class cnrSliderVisitor;
    
//  public:
//    CNR_WildAssGuess(double legal, double sublegal) : CNR(legal, sublegal)  {}

//    virtual double sublegal_ratio(const Fishery& f, const Region& r) const;
//    virtual double legal_ratio(const Fishery& f, const Region& r) const;
//    inline double &get_season_frac(){return cnr_season_fraction;}
    
//  private:
//    double cnr_season_fraction;    
    
//};    

// Season Length method

class CNR_SeasonLength : public CNR {
  public:
    CNR_SeasonLength(double legal, double sublegal, double legal_season, double cnr_season) 
        : CNR(legal, sublegal), legal_season_len(legal_season), cnr_season_len(cnr_season)
    {}

  protected:
    virtual double sublegal_ratio(const Fishery& f, const Region& r) const;
    virtual double legal_ratio(const Fishery& f, const Region& r) const;

    double legal_season_len;
    double cnr_season_len;
};    

// Reported Encounter method

class CNR_ReportedEncounter : public CNR {
  public:
    CNR_ReportedEncounter(double legal, double sublegal, double rpt_legal, double rpt_sublegal,
        double rpt_cat)
    	: CNR(legal, sublegal), rpt_legal_enc(rpt_legal), rpt_sublegal_enc(rpt_sublegal),
          rpt_catch(rpt_cat)
    {}

  protected:
    virtual double sublegal_ratio(const Fishery& f, const Region& r) const;
    virtual double legal_ratio(const Fishery& f, const Region& r) const;

    double rpt_legal_enc;
    double rpt_sublegal_enc;
    double rpt_catch;
};    

#endif
