/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "CeilingParser.h"
#include "CeilingTokens.h"
#include "TopTokens.h"

#include "FisheryManager.h"
#include "FisheryQuotaPolicy.h"


// ctor utility fns
void
CeilingParser::initKeymap()
{
    keymap_.insert(CeilingTokens::CEILING, ceiling);

    // only certain of the generic array specifiers are allowed
    keymap_.insert(GenArrayTokens::YEAR, year);
    keymap_.insert(GenArrayTokens::YEARS, years);
    keymap_.insert(GenArrayTokens::TIME, time);
    keymap_.insert(GenArrayTokens::TIMES, times);
    keymap_.insert(GenArrayTokens::REGION, region);
    keymap_.insert(GenArrayTokens::REGIONS, regions);
    keymap_.insert(GenArrayTokens::FISHERY, fishery);
    keymap_.insert(GenArrayTokens::FISHERIES, fisheries);
    keymap_.insert(Tokens::END1, parseEnd);
    keymap_.insert(Tokens::END2, parseEnd);
}

void
CeilingParser::initState()
{
    // configure the initial state with the correct id list.
    State* state = new State;
    state->context = TopTokens::CEILINGDATA;
    configCeiling(state);
    stateStack.push(state);
}

CeilingParser::CeilingParser(Parser& p)
    : GenArrayParser(p)
{
    initState();
    initKeymap();
}

// access the FisheryPolicy array in the FisheryManager and add the
// list of generic ids to index every element of the array to the
// current state.

void
CeilingParser::configCeiling(State* state)
{
    // create the proper list of generic ids
    FisheryManager::newPolicyTableIndexList(state->ids); 
}


// read in one ceiling object and create FisheryQuotas, placing them
// in the FisheryManager at every generic id in the current list
void
CeilingParser::ceiling()
{
    RWCString forced;
    parseVar(forced);
    double quota;
    parseVar(quota);

    forced.toLower();
    bool forceflag = false;
    if (!strcmp(forced, "forced")) 
        forceflag = true;

    // iterate through the ids in the current state
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        // create a FisheryQuota and add it to the FisheryManager
        auto_ptr<FisheryPolicy> policy =
            auto_ptr<FisheryPolicy>(new FisheryQuotaPolicy(quota, forceflag));
        FisheryManager::insertPolicy(policy, *id);
        ++iter;
    }
    // pop one level of state after processing the data
    parseEnd();
}
