/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "CeilingScalarConfigParser.h"
#include "GenArrayTokens.h"
#include "TopTokens.h"
#include "Tokens.h"

#include "globals.h"

// ctor utility fn

void CeilingScalarConfigParser::initKeymap()
{
    keymap_.insert(GenArrayTokens::YEAR, year);
    keymap_.insert(Tokens::END1, parseEndKeyword);
    keymap_.insert(Tokens::END2, parseEndKeyword);
}

CeilingScalarConfigParser::CeilingScalarConfigParser(Parser& p, const RWCString& fname,
        RWTValOrderedVector<CeilingScalarPolicyControl::CeilingScalarConfig>& configvec)
    : SubParser(p), configvec_(configvec)
{
    initKeymap();

    Fishery* fishery = Fisheries.find(fname);
    // create a CeilingScalarConfigPolicyControl to be configured.
    ceilingConfig_ = auto_ptr<CeilingScalarPolicyControl::CeilingScalarConfig>
        (new CeilingScalarPolicyControl::CeilingScalarConfig(fishery));

    if (!fishery) {
        // log message and continue
        logError(pError(ParseError::BadData, "fishery not found, discarding data", fname,
                 context()).errorMessage());
    }
}

RWCString
CeilingScalarConfigParser::context() const
{
    if (ceilingConfig_->fishery_)
        return(TopTokens::CEILINGSCALAR + ", " + ceilingConfig_->fishery_->get_name());
    else
        return(TopTokens::CEILINGSCALAR);
}

void CeilingScalarConfigParser::year()
{
    int year;
    parseVar(year);
    RWCString yearToken = lastToken(); // for possible error message
    RWCString forced;
    parseVar(forced);
    double scalar;
    parseVar(scalar);
    
    int yearIndex = year - Chronograph::baseYear();
    if (yearIndex >= 0 && yearIndex < Chronograph::nYears()) {
        struct CeilingScalarPolicyControl::CeilingScalarConfig::YearInfo yearInfo;
        yearInfo.active_ = true;
        yearInfo.scalar_ = scalar;
        forced.toLower();
        if (!strcmp(forced, "forced")) 
            yearInfo.force_ = true;
        else 
            yearInfo.force_ = false;
        
        ceilingConfig_->yearConfig_(yearIndex) = yearInfo;
    }
    else {
        logError(pError(ParseError::BadData,
            "ceiling year outside simulation range",
            yearToken, context()).errorMessage());
    }
}


void CeilingScalarConfigParser::parseEndKeyword()
{
    if (ceilingConfig_->fishery_) {
        configvec_.insert(*ceilingConfig_);
    }
    SubParser::parseEndKeyword();
}
