/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// CeilingScalarPolicyControl.h - Class which provides overall control
// for ceiling policy management.

#ifndef CeilingScalarPolicyControl_H
#define CeilingScalarPolicyControl_H

#include "pragma.h"

#include <rw/tvordvec.h>
#include <rw/tvvector.h>
#include "Chronograph.h"
#include "PolicyControl.h"


//class TrackDetailedFisheryCatches;
class Fishery;

// CeilingScalarPolicyControl class - Configured with a list of
// fisheries requiring quotas, and scalar values for the years in
// which the quotas are to occur.  Handles creation of FisheryQuota
// objects.  At the completion of the base period, quota targets are
// computed using the base period catches and the configured quotas.
// FisheryQuota objects are created and inserted into the matrix held
// by the FisheryManager.

class CeilingScalarPolicyControl : public PolicyControl {
  public:
    CeilingScalarPolicyControl(int basePeriodStart, int basePeriodEnd)
        : basePeriodStart_(basePeriodStart), basePeriodEnd_(basePeriodEnd)
    {}

    // managementAction routines where things get done.  this
    // PolicyControl expects to be called after every harvest
    // process invocation, (for the postHarv method).
    virtual void preHarvestManagementAction() {}
    virtual void postHarvestManagementAction();

    class CeilingScalarConfig; // forward decl
    virtual void createQuotas(const CeilingScalarConfig&) const;

    virtual bool valid() const; // confirm that this object is in a valid state

    // class to store the fishery specific ceiling configuration information
    class CeilingScalarConfig {
      public:
        CeilingScalarConfig(const Fishery* f);

        const Fishery* fishery() const
        { return fishery_; }

        struct YearInfo; // forward decl
        const struct YearInfo& yearConfig(int yr) const
        { return yearConfig_(yr); }

        int years() const { return yearConfig_.length(); }

        struct YearInfo {
            bool active_;       // whether there is a ceiling for this year
            bool force_;        // whether to force the ceiling
            double scalar_;     // scalar applied to the base period val to get quota
        };

      private:
        friend class CeilingScalarConfigParser;

        const Fishery* fishery_;
        RWTValVector<struct YearInfo> yearConfig_; // possible ceiling in every year
    }; // end contained class

  protected:
    // parser is a friend
    friend class CeilingScalarParser;

    // base period is specified in calendar years
    int basePeriodStart_;
    int basePeriodEnd_;

    RWTValOrderedVector<CeilingScalarConfig> ceilingConfigs_; // one per fishery
};

#endif // CeilingScalarPolicyControl_H
