/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// cohort class and related global declarations

#ifndef COHORT_H
#define COHORT_H

#include <rw/tpslist.h>
#include <rw/tpordvec.h>
#include <rw/cstring.h>
#include "Age.h"
#include "AbundanceMatrix.h"
#include "HvMort.h"
#include "Bool.h"
#include "Region.h"
#include "RegionList.h"
#include "Chronograph.h"

class Stock;
class TransitionMatrix;
class Chronograph;
class Fishery;

// identifying characteristics of a Cohort.  this class cooperates closely with 
// "GenericArrayIndex", so changes here must be reflected there as well.

class CohortID {
  public:
    // all the default values are 0, for easy use in the GenericArrayIndex
    enum RunType { FALL, SPRING, N_RUN_TYPES };
    enum ProductionType { P_UNKNOWN, WILD, HATCHERY, N_PRODUCTION_STATES };
    enum MaturationStatus { IMMATURE, MATURE, N_MATURATION_STATES };
    enum MarkStatus { M_UNKNOWN, MARKED, UNMARKED, N_MARKS };
    enum TagStatus { T_UNKNOWN, TAGGED, UNTAGGED, N_TAGS };
    enum Sex { S_UNKNOWN, MALE, FEMALE, N_SEXES };
    enum GrowthGroup { G_UNKNOWN, G1, G2, G3, G4, G5, N_GROWTH_GROUPS };

    CohortID(const Stock* stock, unsigned int broodYear, Age age=Age::bottom(),
	   MaturationStatus maturity=IMMATURE,
	   MarkStatus mark=M_UNKNOWN, TagStatus tag=T_UNKNOWN,
       Sex sex=S_UNKNOWN, GrowthGroup growthGroup = G_UNKNOWN);

    CohortID(const CohortID& rhs)
        : stock_(rhs.stock_), broodYear_(rhs.broodYear_), 
        run_(rhs.run_), prodType_(rhs.prodType_),
        maturity_(rhs.maturity_), mark_(rhs.mark_), tag_(rhs.tag_), sex_(rhs.sex_),
        growthGroup_(rhs.growthGroup_), age_(rhs.age_)
    {}

    int operator==(const CohortID& rhs) const
    { return (stock_ == rhs.stock_
	      && broodYear_ == rhs.broodYear_
	      && age_ == rhs.age_
          && run_ == rhs.run_
	      && prodType_ == rhs.prodType_
	      && maturity_ == rhs.maturity_
	      && mark_ == rhs.mark_
          && tag_ == rhs.tag_
	      && sex_ == rhs.sex_
          && growthGroup_ == rhs.growthGroup_
          );
    }

    // return a string representing this cohort id
    const RWCString& idString() const; 
    void clearIdStringCache() const;  // allow the cache to be cleared even if const

    // data access.  note that all write accesses must clear the idStringCache
    const Stock* stock() const      { return stock_; }
    void stock(const Stock* stock)  { stock_ = stock; idString_.resize(0); }

    unsigned int broodYear() const  { return broodYear_; }
    void broodYear(unsigned int broodYear) { broodYear_ = broodYear; idString_.resize(0); }
    
    RunType run() const             { return run_; }
    void run(RunType run)           { run_ = run; idString_.resize(0); }

    ProductionType prodType() const { return prodType_; }
    void prodType(ProductionType prodType) { prodType_ = prodType; idString_.resize(0); }

    MaturationStatus maturity() const { return maturity_; }
    void maturity(MaturationStatus maturity) { maturity_ = maturity; idString_.resize(0); }

    MarkStatus mark() const         { return mark_; }
    void mark(MarkStatus mark)      { mark_ = mark; idString_.resize(0); }

    Sex sex() const                 { return sex_; }
    void sex(Sex sex)               { sex_ = sex; idString_.resize(0); }

    GrowthGroup growthGroup() const { return growthGroup_; }
    void growthGroup(GrowthGroup growthGroup) { growthGroup_ = growthGroup; idString_.resize(0); }

    Age age() const                 { return age_; }

  private:
    friend class Cohort; // allow Cohort direct access
    friend class CohortParser; // parser too

    // defining characteristics
    const Stock* stock_;
    unsigned int broodYear_;
    RunType run_;
    ProductionType prodType_;
    MaturationStatus maturity_;
    MarkStatus mark_;
    TagStatus tag_;
    Sex sex_;
    GrowthGroup growthGroup_;

    // secondary characteristics
    Age age_;

    // cached value of id string
    RWCString idString_;
};

class Cohort {
  public:
    Cohort(const CohortID& id);
    ~Cohort();

    // copy ctor makes a deep copy
    Cohort(const Cohort&);

    int operator== (const Cohort& rhs) const
    { return (this == &rhs); }

    // op== by CohortID
    int operator== (const CohortID& id) const
    { return (id_ == id); }

    // merge the data from another cohort.  only performed if ids match.
    void merge(const Cohort& cohort);

    // accessors
    const CohortID& id() const { return id_; } // note return of reference
    Age age() const { return id_.age_; }
    CohortID::RunType run() const { return id_.run_; }
    CohortID::ProductionType prodType() const { return id_.prodType_; }
    CohortID::MaturationStatus maturity() const { return id_.maturity_; }
    CohortID::MarkStatus markStatus() const { return id_.mark_; }
    CohortID::TagStatus tagStatus() const { return id_.tag_; }
    CohortID::Sex sex() const { return id_.sex_; }
    CohortID::GrowthGroup growthGroup() const { return id_.growthGroup_; }
    bool immature() const { return (id_.maturity_==CohortID::IMMATURE); }
    unsigned broodYear() const { return id_.broodYear_; }
    const Stock* stock() const { return id_.stock_; }
    Age firstHarvestAge() const;
    Age oceanAge() const;
    bool cohortReady(int calendarYear) const;  // is this stock ready to be activated in this year? 

    bool setAge(int calendarYear);

    // abundance manipulations

    // reference current abd for a region
    double& abundance(int loc)
    { return currentAbd_(loc); }
    double abundance(int loc) const
    { return currentAbd_(loc); }

    double& abundance(const Region& loc)
    { return currentAbd_(loc.index()); }
    double abundance(const Region& loc) const
    { return currentAbd_(loc.index()); }

    // current abd all regions
    double abundance() const
    { return sum(currentAbd_); }
    
    // get full current abd vector
    const AbundanceVector& abundanceVec() const
    { return currentAbd_; }

    // reference nat morts
    void clearNaturalMortality()
    { naturalMortality_ = 0; }
    double& naturalMortality(int loc)
    { return naturalMortality_(loc); }
    double naturalMortality(int loc) const
    { return naturalMortality_(loc); }

    // migration
    void migrate(const TransitionMatrix& transMat, const Chronograph& clock);
    
  private:
    const CohortID id_;

    // data
    // current working abundance by region
    AbundanceVector currentAbd_;

    // natural mortality information by region 
    AbundanceVector naturalMortality_;
};

#endif
