/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "DataRequestManager.h"
#include "DataRequestState.h"


// static class members
int DataRequestManager::refcount = 0;
RWTPtrSlist<DataRequest> DataRequestManager::requests;

// singleton instance of the class
static DataRequestManager theDRM;

static void configFn(DataRequest* request, void* )
{  request->config(); }

void DataRequestManager::
configAll()
{
    requests.apply(configFn, 0);
}

static void initFn(DataRequest* request, void* )
{  request->clear(); }

void DataRequestManager::
initAll()
{
    requests.apply(initFn, 0);
}

static void outputFn(DataRequest* request, void* )
{  request->output(); }

void DataRequestManager::
outputAll()
{
    requests.apply(outputFn, 0);
}

void DataRequestManager::
schedule(DataRequest* request)
{
    requests.insert(request);
}

void DataRequestManager::
removeAndDestroy(DataRequest* request)
{
    requests.remove(request);
    delete request;
}

// eventually collecting will have to occur on schedule.  
// for now, just have everyone collect.
static void collectFn(DataRequest* request, void* idPtr)
{  
    SystemProcesses::ProcessId process = *(static_cast<SystemProcesses::ProcessId*>(idPtr));
    request->collect(process); 
}

void DataRequestManager::
collect(SystemProcesses::ProcessId process)
{
    void* processIdPtr = &process;
    requests.apply(collectFn, processIdPtr);
}


// save state by having each DataRequest create its own state object and add them to the list.
// the list owns the objects.
void DataRequestManager::
saveState(DataRequestState& state)
{
    state.stateComponents_.clearAndDestroy();
    RWTPtrSlistIterator<DataRequest> iter(requests);
    DataRequest* request;
    while (request = iter()) {
        State* substate = request->newSaveState();
        if (substate)
            state.stateComponents_.insert(substate);
    }
}
 

// restore state by visiting each state component and calling the restore() method.  the 
// typical implemenation is that the state component has kept a handle to the original
// request, and restores the state directly into the request at this time.

void DataRequestManager::
restoreState(const DataRequestState& state)
{
    RWTPtrSlist<State>::const_iterator ptr = state.stateComponents_.begin();
    while (ptr != state.stateComponents_.end()) {
        (*ptr)->restoreState();
        ++ptr;
    }
}