/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// EnhancedRickerProduction.cpp

#include "EnhancedRickerProduction.h"
#include "RickerProduction.h"
#include "LinearProduction.h"
#include <math.h>

EnhancedRickerProduction::EnhancedRickerProduction(double minSpawners, double maxSpawners, double ev,
                             double rickerA, double rickerB, double age1Recruits,
                             bool densityDependence, double productivityParam,
                             double smoltAtAge1, double maxBroodProportion,
                             double smoltProductionChange)
        : minSpawners_(minSpawners), maxSpawners_(maxSpawners), ev_(ev),
          rickerA_(rickerA), rickerB_(rickerB), age1Recruits_(age1Recruits),
          densityDependence_(densityDependence), smoltAtAge1_(smoltAtAge1),
          maxBroodProportion_(maxBroodProportion), smoltProductionChange_(smoltProductionChange),
          productivityExponent_(exp(productivityParam)), 
          enhancementEfficiency_(densityDependence ? (productivityExponent_ / exp(rickerA)) : 0)
{}

// spawning function.  computes production via ricker with enhancement
// and optional density dependence.

double EnhancedRickerProduction::
spawn(double escapement) const
{
    if (escapement < minSpawners_)
        return 0.0;

    // compute enhancement spawners, limited by maximum brood size
    double enhancementSpawners = (smoltProductionChange_ * smoltAtAge1_) / productivityExponent_;
    double maxBrood = escapement * maxBroodProportion_;
    if (enhancementSpawners > maxBrood)
        enhancementSpawners = maxBrood;

    // compute portion of natural escapement
    double naturalEscapement = escapement - enhancementSpawners;
    if (naturalEscapement > maxSpawners_)
        naturalEscapement = maxSpawners_;

    if (!densityDependence_) {
        // density independent.  create a ricker production for the natural portion,
        // and a linear for the enhancement portion, and sum the results.
        RickerProduction naturalProd(minSpawners_, naturalEscapement, ev_, rickerA_,
                                     rickerB_, age1Recruits_);
        LinearProduction enhancedProd(naturalEscapement, escapement, ev_, productivityExponent_);

        return naturalProd.spawn(escapement) + enhancedProd.spawn(escapement);
    }
    else {
        // density dependent.  one ricker function with a modified input.
        enhancementSpawners *= enhancementEfficiency_ * age1Recruits_;

        // compute a new total escapement level that reflects increased efficiency.
        escapement = naturalEscapement + enhancementSpawners;

        RickerProduction totalProd(minSpawners_, maxSpawners_, ev_, rickerA_, rickerB_, age1Recruits_);
        
        return totalProd.spawn(escapement);
    }
}

