/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// Fishery class implementations

#include "SystemClock.h"
#include "Fishery.h"
#include "HarvestManager.h"
#include "Stock.h"
#include "Region.h"
#include "Geographer.h"
#include "HvMort.h"
#include "Cohort.h"
#include "FisheryUnit.h"
#include "Fisherymanager.h"
#include "GenericArray.h"
#include "CNR.h"
#include "Shaker.h"

#include <rw/math/mathvec.h>
#include <rw/cstring.h>


Fishery::Fishery(const RWCString& nm) : name(nm), gear(TROLL), encounterRate_(0), index_(0)
{}


Fishery::~Fishery()
{
    fisheryUnitPool_.clearAndDestroy();
}

void Fishery::config()    
{
    // this config requires others to be properly set
    Geographer::checkConfig();
    Chronograph::checkConfig();

    // set up new memory pool for fishery units
    fisheryUnitPool_.clearAndDestroy();
    for (int i=0; i < Geographer::nRegions(); ++i) {
        fisheryUnitPool_.append(new FisheryUnit(*this, Geographer::region(i)));
    }
    // activate the fishery
    setDefaultSchedule();
}

// by default the fishery is "always on".  since this is called from "config", it also shapes the array
void Fishery::
setDefaultSchedule()
{
    activitySchedule_.reshape(Chronograph::nYears(), Chronograph::nTimeSteps(), Geographer::nRegions());
    for (int i=0; i < Chronograph::nYears(); ++i) {
        for (int j=0; j < Chronograph::nTimeSteps(); ++j) {
            for (int k=0; k < Geographer::nRegions(); ++k) {
                activitySchedule_[i][j][k] = false;
            }
        }
    }
}

void Fishery::
clearSchedule()
{
    for (int i=0; i < Chronograph::nYears(); ++i) {
        for (int j=0; j < Chronograph::nTimeSteps(); ++j) {
            for (int k=0; k < Geographer::nRegions(); ++k) {
                activitySchedule_[i][j][k] = false;
            }
        }
    }
}

void Fishery::
activate(int year, int time, int region)
{
    activitySchedule_[year][time][region] = true;
}

// helper fn clear a FisheryUnit in the list
static void clearFisheryUnit(FisheryUnit* fu, void *)
{   fu->clear(); }

// clean out the FisheryUnits currently in use and empty the list
void Fishery::
clearFisheryUnits()
{
    fisheryUnits_.apply(clearFisheryUnit, 0);
    fisheryUnits_.clear();
}

// generate the list of currently appropriate hvmorts.  this must be
// done at the beginning of each timestep!
void Fishery::makeHarvestList()
{
    // first empty the old fishery unit list
    clearFisheryUnits();

    // then make a new one, using memory from the pool
    int year = ::SystemClock->currentYearIndex();
    int time = ::SystemClock->currentTimeIndex();
    for (int i=0; i < Geographer::nRegions(); ++i) {
        if (activitySchedule_[year][time][i]) {
            fisheryUnits_.insert(fisheryUnitPool_[i]);
        }
    }

    // clear the local harvest list (stale pointers); the FisheryUnit owned the pointed to objects
    harvests_.clear(); 

    // have the FisheryUnits generate new HvMort lists
    for (i=0; i < fisheryUnits_.entries(); ++i) {
        fisheryUnits_(i)->makeHarvestList();
        // update the local list of HvMort objects with the appropriate
        // ones for the new timestep.  
        fisheryUnits_(i)->getHarvests(harvests_);
    }
}

// relative harvest rate is current policy scalar, capped at 1.0, for CNRs
double Fishery::relHr(const Region& region) const
{ 
    GenericArrayIndex gendex(0, index(), region.index(), 
        ::SystemClock->currentYearIndex(), ::SystemClock->currentTimeIndex());
    double scalar = FisheryManager::effortScalar(gendex);
    return (scalar < 1.0) ? scalar : 1.0; 
} 

// helper fn to call the harvest() fn
static void setHarvestCNR(HvMort* hvm, void *)
{
    hvm->setCNR();
}

// time independent harvesting
void Fishery::setCNR()
{
    harvests_.apply(setHarvestCNR, 0);
}

// shaker morts
void Fishery::setShakers()
{ 
	GenericArrayIndex gendex(0, index(), 0, 
        ::SystemClock->currentYearIndex(), ::SystemClock->currentTimeIndex());
	const Shaker* shaker = HarvestManager::shaker(gendex);
    if (shaker)
        shaker->setShakers(this); 
}

double Fishery::
subLegalCnrMortRate(const Region& region) const
{ 
	GenericArrayIndex gendex(0, index(), region.index(), 
        ::SystemClock->currentYearIndex(), ::SystemClock->currentTimeIndex());
	const Shaker* shaker = HarvestManager::shaker(gendex);
    if (shaker)
        return shaker->totalSubLegalCnrMortRate();
    else 
        return 0.0;
}

double Fishery::
dropOffMortRate(const Region& region) const
{ 
	GenericArrayIndex gendex(0, index(), region.index(), 
        ::SystemClock->currentYearIndex(), ::SystemClock->currentTimeIndex());
	const Shaker* shaker = HarvestManager::shaker(gendex);
    if (shaker)
        return shaker->dropOffMortRate();
    else 
        return 0.0;
}

// helper fn to call harvest() in FisheryUnit
static void harvest(FisheryUnit* fu, void *)
{   fu->harvest(); }

// time independent harvesting.  the fishery tells all fisheryUnits to harvest.  

void Fishery::takeHarvests()
{
    fisheryUnits_.apply(harvest, 0);
}

// actually apply the mortalities to the cohort abundances
static void applyHarvests(HvMort* hvmort, void *) 
{   hvmort->applyHarvestMorts(); }

void Fishery::applyHarvestMorts()
{
    harvests_.apply(applyHarvests, 0);
}

// total catch in the current timestep
double Fishery::legalCatch() const
{
    double tcatch = 0;
    for (int i=0; i < harvests_.length(); ++i)
        tcatch += harvests_(i)->legalCatch();
    
    return tcatch;
}

// total catch by region
double Fishery::legalCatch(const Region& region) const
{
    for (int i=0; i < fisheryUnits_.entries(); ++i) {
        if (fisheryUnits_(i)->region() == region)
            return fisheryUnits_(i)->legalCatch();
    }
    // only gets here if the region not harvested in this fishery
    return 0.0;
}

    
