/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

#include "Fisheries.h"
#include "SystemClock.h"
#include "Geographer.h"
#include "FisheryManager.h"
#include "FixedHarvestRatePolicy.h"
#include "FisheryUnit.h"
#include "PolicyControlManager.h"

auto_ptr<PolicyTable> FisheryManager::policyTable_ = auto_ptr<PolicyTable>(0);

// config routine must be called after the basic system configs are completed.
void FisheryManager::config()
{
    policyTable_ = auto_ptr<PolicyTable>(new PolicyTable(::Fisheries.num(), Geographer::nRegions(), 
        Chronograph::nYears(), Chronograph::nTimeSteps()));

    // fill in table with default FixedHarvestRatePolicy
    for (unsigned f=0; f<::Fisheries.num(); ++f) {
        for (unsigned r=0; r<Geographer::nRegions(); ++r) {
            for (unsigned y=0; y<Chronograph::nYears(); ++y) {
                for (unsigned t=0; t<Chronograph::nTimeSteps(); ++t) {
                    GenericArrayIndex index(0, f, r, y, t);
                    (*policyTable_)[index] = auto_ptr<FisheryPolicy>(new FixedHarvestRatePolicy);
                }
            }
        }
    }
}


// insert a new policy in the table
void FisheryManager::
insertPolicy(auto_ptr<FisheryPolicy> policy, const GenericArrayIndex& gendex)
{
    (*policyTable_)[gendex] = policy;
}


// return the proper FisheryPolicy for this FisheryUnit in the current year and timestep

FisheryPolicy* FisheryManager::
findPolicy(FisheryUnit* fisheryUnit)
{
    GenericArrayIndex gendex(0, fisheryUnit->fishery().index(), fisheryUnit->region().index(),
        *::SystemClock);

    // look up the policy from the table, retrieve the raw ptr from the auto ptr, and return it
    return (*policyTable_)[gendex].get();
}

// return the current effort scalar
double FisheryManager::effortScalar(const GenericArrayIndex& gendex)
{
    return ((*policyTable_)[gendex]->effortScalar());
}

// routine to walk the policy lists and perform the harvests

void FisheryManager::takeHarvests() 
{
    // clear harvests and initialize the harvest lists for this timestep
    for (int i=0; i < Fisheries.num(); ++i) {
        Fisheries[i].makeHarvestList();
    }
    // give the PolicyControls a chance to act
    PolicyControlManager::preHarvestManagement();

    // perform the harvesting
    for (i=0; i < Fisheries.num(); ++i) {
        Fisheries[i].takeHarvests();
     	Fisheries[i].setShakers();
     	Fisheries[i].setCNR();
    }

    // give the PolicyControls another chance to act, possibly invoking more harvesting
    PolicyControlManager::postHarvestManagement();

    // update the state with the new harvests
    for (i=0; i < Fisheries.num(); ++i)
        Fisheries[i].applyHarvestMorts();
}