/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// FisheryQuotaPolicy.cpp

#include "FisheryQuotaPolicy.h"
#include "FisheryUnit.h"
#include "HvMort.h"
#include "Fishery.h"
#include "Search.h"
#include "log.h"
#include <math.h>

const double FisheryQuotaPolicy::tolerance_ = 0.00001;
const double FisheryQuotaPolicy::xtolerance_ = 0.0001;
const int FisheryQuotaPolicy::maxIter_ = 1000;

void FisheryQuotaPolicy::
harvest(FisheryUnit* fisheryUnit)
{
    // first make one harvest pass with the current scalar
    fisheryUnit->harvest(effortScalar_);
    
    // grab the total catch
    double unitCatch = fisheryUnit->legalCatch();
    
    // check for meeting initial conditions
    if (unitCatch > quota_ || forceFlag_) {

        // create and initialize search object
        Search search(quota_, tolerance_, xtolerance_, maxIter_);
        search.initialGuess(effortScalar_);
        
        // conduct search
        while (!search.done(unitCatch)) {
            effortScalar_ = search.nextGuess(unitCatch);
            fisheryUnit->harvest(effortScalar_);
            unitCatch = fisheryUnit->legalCatch();
        }
        if (search.stopCode() != Search::SUCCESS) {
            switch (search.stopCode()) {
            case Search::MAXITER:
                ::LogMsg(L_Error, "maximum number of iterations exceed in quota search for %s\n",
                    fisheryUnit->fishery().get_name());
                break;
            case Search::FSTALL:
                ::LogMsg(L_Error, "not enough available fish to acheive quota in %s\n",
                    fisheryUnit->fishery().get_name());
                break;
            case Search::INITF:
                ::LogMsg(L_Error, "cannot force ceiling in %s due to initial catch of 0.0\n",
                    fisheryUnit->fishery().get_name());
            }
        }
    }
}
