/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#ifndef FisheryUnit_H
#define FisheryUnit_H

// FisheryUnit - the intersection of a fishery and region, with list
// of active years and timesteps.  the FisheryUnit owns and maintains 
// the list of HvMort objects currently active.  at the beginning of 
// each timestep, the list is emptied and recreated.  note that Fisheries
// may keep pointers into this list, and are thus responsible for 
// removing all such pointers at the beginning of each timestep.

#include "pragma.h"

#include <rw/tpordvec.h>
#include <rw/tvslist.h>

#include "Range.h"
#include "Chronograph.h"

class HvMort;
class Fishery;
class Region;

class FisheryUnit {
  public:
    FisheryUnit(const Fishery& fishery, const Region& region) 
        : fishery_(fishery), region_(region) {}
    ~FisheryUnit() { harvests_.clearAndDestroy(); }

    const Fishery& fishery() const { return fishery_; }
    const Region& region() const { return region_; }

    // make a list of all appropriate harvests
    void makeHarvestList();
    // add harvests to the passed in list
    void getHarvests(RWTPtrOrderedVector<HvMort>& harvests) const;

    // main entry point to perform all harvesting
    void harvest();
    // entry point used by FisheryPolicy to perform all harvesting with a given
    // effor scalar
    void harvest(double effortScalar);

    // return total legal catch
    double legalCatch() const;

    // empty this FisheryUnit
    void clear()
    { harvests_.clearAndDestroy(); }

  private:
    const Fishery& fishery_;
    const Region& region_;
    
    RWTPtrOrderedVector<HvMort> harvests_; 
};

#endif // FisheryUnit_H
