/**********************************************************************
  Columbia Basin Research Software Library Tools
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// FreeFormReader implementations

#include "ParseError.h"
#include "FreeFormReader.h"

// normal ctor
FreeFormReader::FreeFormReader(const RWCString& fileName,
							   const char* separators, 
							   const char* operators)
{
    // FreeFormReader handles comments and quoted strings, so it
	//  also adds the # and " characters to the list of operators.
    // get the first token from the file into the nextToken_ holder
    if (operators)
        myOperators_ = operators;
	myOperators_.append( "#\"" );
	{
		RCPtr<SharedTokenBuffer> tempTokenBuf(
				new SharedTokenBuffer( fileName, separators,
						  const_cast<char *>(myOperators_.data()) ) );
		tokenBuf_ = tempTokenBuf;
		// tempTokenBuf gets destructed.
	}

    nextToken();
}

// advance to the beginning of the next line, skipping comments and
// lines without tokens.  a recognized comment causes the rest of the
// line to be skipped.  return null on EOF.
const char*
FreeFormReader::nextDataLine()
{
    while (!tokenBuf_->currentToken_.value_.isNull()   // detects beginning and end of file
        && tokenBuf_->currentToken_.lineNum_ == tokenBuf_->nextToken_.lineNum_) {
    
        nextToken();
    }
    return tokenBuf_->currentLine();
}

// return next token to the user.  advance the buffer to get a new "peek" token
// into the lookahead.

RWCString FreeFormReader::nextToken()
{
    tokenBuf_->currentToken_ = tokenBuf_->nextToken_;
    tokenBuf_->nextToken_.value_ = advanceToken();
    tokenBuf_->nextToken_.lineNum_ = tokenBuf_->lineCount();
    return tokenBuf_->currentToken_.value_;
}

// advance line buffer and retrieve next token string, skipping whitespace (including
// line returns) and comments.  recognized comments cause the rest of the 
// line to be discarded.  also return a quoted string as a single token, with
// the quotes stripped.  return null on EOF.

RWCString
FreeFormReader::advanceToken()
{
    RWCString tokenStr = tokenBuf_->nextLineToken();
    while (tokenStr.isNull() || isComment_(tokenStr)) { // end of line or comment
        if (!tokenBuf_->nextLine())
            return RWCString();	// end of file found
        else
            tokenStr = tokenBuf_->nextLineToken();
    }
    // check for string
    if (tokenStr(0) == '"') {
        // change definition of whitespace, get entire string, 
        // restore whitespace, discard trailing quote
        tokenBuf_->setSeparators("\n", "\"");
        tokenStr = tokenBuf_->nextLineToken();
        tokenBuf_->restoreSeparators();
        RWCString tmp = tokenBuf_->nextLineToken();
        // make sure there was a trailing quote on this line, else throw exception
        if (tmp.isNull() || tmp(0) != '"')
            throw ParseError(tokenBuf_->fileName(), tokenBuf_->lineCount(), ParseError::BadData,
                "missing \" at end of string", tokenStr);
    }
    return tokenStr;
}

