/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// GenericArray.h

#ifndef GenericArray_H
#define GenericArray_H

#include "pragma.h"

//#include <rw/tpslist.h>
#include <rw/tpordvec.h>
#include "RCObject.h"
#include "RCPtr.h"

class Chronograph;
class Cohort;

// GenericArray is a base class providing a simple array interface for
// reading and writing values in an array.  the dimensionality of the
// underlying array is not known, so the indexing is done with a
// GenericArrayIndex object.
// GenericArrays are also inherently reference-countable.

class GenericArrayIndex;

template <class T>
class GenericArray {
  public:
    virtual ~GenericArray() {}
    
    // access to array elements
    virtual const T& operator[](const GenericArrayIndex&) const = 0;
    virtual T& operator[](const GenericArrayIndex&) = 0;

    // append to a list of generic indices which access every element
    // in the array.  note that the indices are created on the heap,
    // and ownership is passed to the caller.  this list may be used
    // as a crude sort of iterator over the entire array.
    virtual void newGenericIndexList(RWTPtrOrderedVector<GenericArrayIndex>&) const = 0;
};


class GenericArrayIndex {
  public:
    GenericArrayIndex();
    GenericArrayIndex(const Cohort* c, unsigned fishery=0, unsigned region=0, 
                      unsigned year=0, unsigned time=0);
    GenericArrayIndex(const Cohort*c, unsigned fishery, unsigned region, const Chronograph& clock);
    GenericArrayIndex(const Chronograph& clock);
    GenericArrayIndex(unsigned year, unsigned time);

    int operator==(const GenericArrayIndex& rhs) const;
    
    // sets the GenericArrayIndex to contain the size values for all the fields
    void resetToSizes();

    // data items which are used to identify a particular array index
    // in a GenericArray.  items known to be not relevant to a
    // particular array may be left blank.

    // NOTE: if new identifying characteristics are added to the CohortID definition, 
    // then equivalent changes must be made here and in GenericArray.cpp
    
    unsigned stockIndex;
    unsigned broodYear;
    unsigned run;
    unsigned prodType;
    unsigned maturityIndex; 
    unsigned mark;
    unsigned tag;
    unsigned sex;
    unsigned growthGroup;
    unsigned age;

    unsigned fisheryIndex;
    unsigned regionIndex;

    unsigned yearIndex;
    unsigned timeIndex;
};

#endif // GenericArray_H
