/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include <rw/tpslist.h>
#include <rw/tpordvec.h>
#include "RegionList.h"
#include "Fishery.h"
#include "FisheryUnit.h"
#include "Cohort.h"
#include "CohortManager.h"
#include "HarvestProcess.h"
#include "FisheryPolicy.h"
#include "FixedHarvestRatePolicy.h"
#include "Shaker.h"
#include "CNR.h"
#include "globals.h"
#include "HarvestManager.h"
#include "HvMort.h"
#include "GenericArray.h"
#include "GenericDefaultArray.h"

// static definitions

const double HarvestManager::defaultHvRate_ = 0.0;
const double HarvestManager::defaultFp_ = 1.0;
const double HarvestManager::defaultPnv_ = 0.0;

auto_ptr<HvRateTable> HarvestManager::hvRateTable_ 
  = auto_ptr<HvRateTable>(new GenericDefaultArray<double>(defaultHvRate_));

auto_ptr<FpTable> HarvestManager::fpTable_ 
  = auto_ptr<FpTable>(new GenericDefaultArray<double>(defaultFp_));

auto_ptr<PnvTable> HarvestManager::pnvTable_ 
  = auto_ptr<PnvTable>(new GenericDefaultArray<double>(defaultPnv_));

// this one ensures that the ShakerTable exists, but puts a null auto ptr in by default
auto_ptr<ShakerTable> HarvestManager::shakerTable_ 
  = auto_ptr<ShakerTable>(new GenericDefaultArray<auto_ptr<Shaker> >(auto_ptr<Shaker>(0)));

// see comment for ShakerTable above
auto_ptr<CNRTable> HarvestManager::cnrTable_ 
  = auto_ptr<CNRTable>(new GenericDefaultArray<auto_ptr<CNR> >(auto_ptr<CNR>(0)));


// reset all the tables to default values.
void HarvestManager::
config()
{
    // memory management handled by auto_ptr
    hvRateTable_ = auto_ptr<HvRateTable>(new GenericDefaultArray<double>(defaultHvRate_));
    fpTable_ = auto_ptr<FpTable>(new GenericDefaultArray<double>(defaultFp_));
    pnvTable_ = auto_ptr<PnvTable>(new GenericDefaultArray<double>(defaultPnv_));

    // this one ensures that the ShakerTable exists, but inserts a null auto ptr by default
    shakerTable_ = auto_ptr<ShakerTable>(new GenericDefaultArray<auto_ptr<Shaker> >(auto_ptr<Shaker>(0)));

    // see comment for ShakerTable above
    cnrTable_ = auto_ptr<CNRTable>(new GenericDefaultArray<auto_ptr<CNR> >(auto_ptr<CNR>(0)));
}


// routines for inserting new tables and values

void HarvestManager::
configHvRates(auto_ptr<HvRateTable> newTable)
{
    hvRateTable_ = newTable;	// assume ownership, auto_ptr deletes any old table
}


void HarvestManager::insertHvRate(double hvRate, const GenericArrayIndex& index)
{
    (*hvRateTable_)[index] = hvRate;
}

void HarvestManager::
configFps(auto_ptr<FpTable> newTable)
{
    fpTable_ = newTable;	// assume ownership, auto_ptr deletes any old table
}

void HarvestManager::insertFp(double fp, const GenericArrayIndex& index)
{
    (*fpTable_)[index] = fp;
}

void HarvestManager::
configPnvs(auto_ptr<PnvTable> newTable)
{
    pnvTable_ = newTable;	// assume ownership, auto_ptr deletes any old table
}

void HarvestManager::
insertPnv(double pnv, const GenericArrayIndex& index)
{
    (*pnvTable_)[index] = pnv;
}

void HarvestManager::
configShakers(auto_ptr<ShakerTable> newTable)
{
    shakerTable_ = newTable;  // assume ownership, auto_ptr deletes any old table
}

void HarvestManager::
insertShaker(auto_ptr<Shaker> shaker, const GenericArrayIndex& index)
{
    (*shakerTable_)[index] = shaker;  // assume ownership
}

void HarvestManager::
configCNRs(auto_ptr<CNRTable> newTable)
{
    cnrTable_ = newTable;  // assume ownership, auto_ptr deletes any old table
}

void HarvestManager::
insertCNR(auto_ptr<CNR> cnr, const GenericArrayIndex& index)
{
    (*cnrTable_)[index] = cnr;  // assume ownership
}

// create HvMort objects for the FisheryUnit supplied, adding them to the 
// passed in list, which assumes ownership.  it is assumed that new hvmort
// objects are properly zeroed.

void HarvestManager::
createHarvestMorts(const FisheryUnit* unit, RWTPtrOrderedVector<HvMort>& mortlist)
{
    // the active cohort list should be obtained from the CohortManager, but
    // at the moment it resides in the Stocks themselves.
    for (int i=0; i < ::Stocks.num(); ++i) {
        const CohortList& cohorts = Stocks[i].getCohorts();
        CohortList::const_iterator cohortPtr = cohorts.begin();
        while (cohortPtr != cohorts.end()) {
            if ((*cohortPtr)->age() >= (*cohortPtr)->firstHarvestAge()
                && (*cohortPtr)->abundance(unit->region()) > 0.0) {
                // temporarily use LinearHarvestProcess; make configurable later
                auto_ptr<const HarvestProcess> harvestProcess(new LinearHarvestProcess); 
                mortlist.append(new HvMort(**cohortPtr, unit->fishery(), unit->region(), 
                    harvestProcess));
            }
            ++cohortPtr;
        }
    }
}

// access harvest rate data
// base harvest rate
double HarvestManager::
baseHr(const HvMort* hvmort)
{
    GenericArrayIndex id(&(hvmort->cohort()), hvmort->fishery().index(), hvmort->region().index(),
			 ::SystemClock->currentYearIndex(), ::SystemClock->currentTimeIndex());

    return ((*hvRateTable_)[id]);
}

// base harvest rate * fp
double HarvestManager::
hvRate(const HvMort* hvmort)
{
    GenericArrayIndex id(&(hvmort->cohort()), hvmort->fishery().index(), hvmort->region().index(),
			 ::SystemClock->currentYearIndex(), ::SystemClock->currentTimeIndex());

    return ((*hvRateTable_)[id] * (*fpTable_)[id]);
}

// retrieve pnv
double HarvestManager::
pnv(const HvMort* hvmort)
{
    GenericArrayIndex id(&(hvmort->cohort()), hvmort->fishery().index(), hvmort->region().index(),
			 ::SystemClock->currentYearIndex(), ::SystemClock->currentTimeIndex());
    return (*pnvTable_)[id];
}

// pv is 1 - pnv
double HarvestManager::
pv(const HvMort* hvmort)
{
    return 1.0 - pnv(hvmort);
}

// access shaker objects
const Shaker* HarvestManager::
shaker(const GenericArrayIndex& gendex)
{
    return ((*shakerTable_)[gendex].get());
}

// access cnr objects
const CNR* HarvestManager::
cnr(const GenericArrayIndex& gendex)
{
    return ((*cnrTable_)[gendex].get());
}
