/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// HarvestManager.h - system monostate.  serves as a factory for
// HvMort objects, and a data repository for harvest rate information.

#ifndef HarvestManager_H
#define HarvestManager_H

#include "auto_ptr.h"

class Cohort;
class HvMort;
class FisheryUnit;
class FisheryPolicy;
class Shaker;
class CNR;

template <class T> class RWTPtrSlist;
template <class T> class RWTPtrOrderedVector;

template <class  T> class GenericArray;
class GenericArrayIndex;

typedef GenericArray<double> HvRateTable;
typedef GenericArray<double> FpTable;
typedef GenericArray<double> PnvTable;
typedef GenericArray<auto_ptr<Shaker> > ShakerTable;
typedef GenericArray<auto_ptr<CNR> > CNRTable;

// precondition: this class is not in a valid state until all of the
// "config()" methods have been called to establish the data.

class HarvestManager {
  public:
    static void config(); // configure the harvest manager

    // accessors
    static double baseHr(const HvMort*);  // base harvest rate
    static double hvRate(const HvMort*);  // base harvest rate * fp
    static double pnv(const HvMort*);
    static double pv(const HvMort*);
	static const Shaker* shaker(const GenericArrayIndex&);
	static const CNR* cnr(const GenericArrayIndex&);
	
    // create all appropriate hvmort objects for the given FisheryUnit, and
    // attach to the given list, which retains ownership.  
    static void createHarvestMorts(const FisheryUnit* unit, RWTPtrOrderedVector<HvMort>& mortlist);
    static void zeroHarvests();

  private:
    friend class HarvestParser;
    friend class FpParser;
    friend class PnvParser;
	friend class ShakerParser;
	friend class CNRParser;

    // harvest rate data config
    static void configHvRates(auto_ptr<HvRateTable> newTable);
    static void insertHvRate(double hvRate, const GenericArrayIndex& index);
    
    // fp data config
    static void configFps(auto_ptr<FpTable> newTable);
    static void insertFp(double fp, const GenericArrayIndex& index);

    // pnv data config
    static void configPnvs(auto_ptr<PnvTable> newTable);
    static void insertPnv(double pnv, const GenericArrayIndex& index);

    // shaker data config
    static void configShakers(auto_ptr<ShakerTable> newTable);
    static void insertShaker(auto_ptr<Shaker> shaker, const GenericArrayIndex& index);

    // cnr data config
    static void configCNRs(auto_ptr<CNRTable> newTable);
    static void insertCNR(auto_ptr<CNR> cnr, const GenericArrayIndex& index);

    // data
    static const double defaultHvRate_;
    static const double defaultFp_;
    static const double defaultPnv_;
    
    static auto_ptr<HvRateTable> hvRateTable_;
    static auto_ptr<FpTable> fpTable_;
    static auto_ptr<PnvTable> pnvTable_;
    static auto_ptr<ShakerTable> shakerTable_;
    static auto_ptr<CNRTable> cnrTable_;
};

#endif // HarvestManager_H
