/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// IterationControl.h - this class manages the data required to cause 
// an iteration over a specific range of timesteps.  there may be a number
// of objects of this class all operating simultaneously.

// IterationControl is an abstract base class.

#ifndef IterationControl_H
#define IterationControl_H

#include "Chronograph.h"
#include "SystemState.h"
#include "auto_ptr.h"


// An IterationControl keeps track of the beginning and ending
// timesteps (including year) of iteration in variables firstStep_ and
// lastStep_.  The saved SystemState is provided by the user, and must
// correspond to the state at the firstStep_.  A method for the user
// to delete the saved state is provided as a space optimization.
//
// The basic functionality is provided by the test
// iterationRequired(), which determines if the current timestep is
// the lastStep_, and then if any iteration is called for.  This is
// accomplished through a call to the pure virtual
// endConditionSatisfied().  If iterationRequired() returns 'true', it
// is expected that resetControlVariables() (also pure virtual) will
// be called at some time shortly thereafter, as part of the
// initialization of the new iteration.  It is also up to the user to
// reset the clock to the firstStep_, and to restore the state
// (possibly using the restoreState() method provided).

class IterationControl {
  public:
    IterationControl(const TimeStamp& first, const TimeStamp& last)
        : firstStep_(first), lastStep_(last)
    {}

    // public abstract interface
    virtual void init() = 0;           // per-simulation initializations
    virtual bool iterationRequired();  // default method calls endConditionSatisfied()
    virtual void resetControlVariables() = 0;
    
    // accessors
    const TimeStamp& firstStep() const { return firstStep_; }
    const TimeStamp& lastStep() const { return lastStep_; }
    
    // state manipulation
    virtual void saveState(auto_ptr<const SystemState> state) { saveState_ = state; }
    virtual void restoreState() const { saveState_->restoreState(); }
    virtual void deleteState() { saveState_.reset(); }
    
  protected:
    virtual bool endConditionSatisfied() = 0;
    
    TimeStamp firstStep_;
    TimeStamp lastStep_;
    auto_ptr<const SystemState> saveState_;
};

#endif // IterationControl_H
