/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// IterationManager.h - manages iteration in the system by tracking
// and analyzing IterationControl objects.
// This is a system monostate (all static methods and data).

#ifndef IterationManager_H
#define IterationManager_H

#include "pragma.h"

#include "IterationControl.h"
#include "DestructiveTPtrSlist.h"
#include "DestructiveTPtrSortedDlist.h"
#include "auto_ptr.h"

RW_SL_STD_USING  // brings in stdlib namespace when needed

// beginTimeStep() is called as the first action in every timestep.  
// it may record data such as the system state, and it may reset the 
// state and the system clock.  endTimeStep() is called at the end of 
// every timestep, and makes decisions which may result in an iteration
// the next time beginTimeStep() is called.


class IterationManager {
  public:
    static void config();        // reconfigure system
    static void init();          // reinitialize for new simulation

    static void beginTimeStep(); // called at the start of every timestep
    static void endTimeStep();   // called at the end of every timestep

    static void insertIterationControl(auto_ptr<IterationControl> control);
    
  private:
    // IterationManager is the repository for IterationControl objects.
    // these need to be kept grouped by firstStep along with some
    // ancillary information for each group
    class ControlGroup {
      public:
        ControlGroup(auto_ptr<IterationControl> control);
        
        void insertControl(auto_ptr<IterationControl> control);

        // public data for use by the owner class
        TimeStamp firstStep;
        TimeStamp latestLastStep;
        DestructiveTPtrSlist<IterationControl> controls;
    };
    class ControlGroupComparator {
      public:
        // comparison is based on firstStep;
        bool operator()(const ControlGroup& x, const ControlGroup& y) const
        { return (x.firstStep < y.firstStep) ? true : false; }
    };
    typedef DestructiveTPtrSortedDlist<ControlGroup, ControlGroupComparator> ControlGroupDlist; 
    typedef RWTPtrSortedDlistIterator<ControlGroup, ControlGroupComparator> ControlGroupIter; 

    // helper find fn
    static bool findFirstStepGroup(const ControlGroup* group, void* data);

    // repository for all IterationControls, grouped and ordered by firstStep
    static ControlGroupDlist controlGroups_;

    // iteration requests pending at any given time; pointers into controlGroups_
    static RWTPtrSlist<IterationControl> pendingRequests_;
};


#endif // IterationManager_H
