/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// Lex.cpp - class to tokenize a string

#include "Lex.h"

char* Lex::NULLSTRING = "";

Lex::Lex(const char* str, const char* separators, const char* operators)
  : buf_(str), separators_(separators), operators_(operators), 
    ctorSeparators_(separators), ctorOperators_(operators)
{
    terminators_[TERMBUFSIZE-1] = 0;
    initTerminators();
    initBuf();
}

void Lex::
initTerminators()
{
    if (!separators_)
        separators_ = " \t\n";

    if (!operators_)
        operators_ = "#\"";

    strncpy(terminators_, separators_, TERMBUFSIZE-1);
    strncat(terminators_, operators_, TERMBUFSIZE-1 - strlen(separators_));
}

// initialize buffer string and consume leading separators
void Lex::
initBuf()
{
    if (!buf_) 
        buf_ = NULLSTRING;

    bufOffset_ = tokenOffset_ = 0;
    eatSeparators();
}

// set buf_ to new string to tokenize.  don't change separators or operators.
void Lex::
lexString(const char* str)
{
    buf_ = str;
    initBuf();
}

// set new separators and operators
void Lex::
setSeparators(const char* separators, const char* operators)
{
    separators_ = separators;
    operators_ = operators;
    initTerminators();
}

// restore separators and operators to original values
void Lex::
restoreSeparators()
{
    setSeparators(ctorSeparators_, ctorOperators_);
}

// determine if next token is an operator
bool Lex::
atOperator() const
{
    return (0 != strchr(operators_, buf_[bufOffset_]));
}

// consume whitespace
void Lex::
eatSeparators()
{
    int seplen = strspn(&buf_[bufOffset_], separators_);
    bufOffset_ += seplen;
}

// return next token - precondition is that separators are already consumed
char* Lex::
token()
{
    if (endTest())
        return 0;

    tokenOffset_ = 0;
    int tokenlen = 0;

    // determine length of token - special case for operators
    if (atOperator())
        tokenlen = 1;
    else {
        tokenlen = strcspn(&buf_[bufOffset_], terminators_);
        if (tokenlen >= TOKENBUFSIZE)
            tokenlen = TOKENBUFSIZE-1;
    }

    // copy token to token buffer
    for (int i=0; i < tokenlen; ++i)
        token_[tokenOffset_++] = buf_[bufOffset_++];

    token_[tokenOffset_] = 0;

    eatSeparators();
    return token_;
}