/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// Lex.h - class to tokenize a string

#ifndef Lex_H
#define Lex_H

#include "pragma.h"

#include "Bool.h"
#include <string.h>

// Lex is initialized with a string to analyze and optionally with strings specifying
// separators and single character operators.  Successive calls to token() return the next
// token in the string.  default separators are whitespace (" \t\n") and default operators
// are '#' and '"'.  the token is returned as a reference to internal storage
// kept by this class, which remains valid only while the Lex object is in scope and
// up until the next call to token().  token() returns null when the string is exhausted.
// the user of this class is responsible for the storage for the string to be analyzed and
// for the separator and operator strings if supplied, and for ensuring that all remain valid 
// and unchanged for the lifetime of the Lex object.  Special entry points are supplied for
// specifying new strings for existing Lex objects if desired.

// class invariants:
//   - leading separators have always already been consumed upon entry to method token().
//   - terminators_ is always the concatenation of separators_ and operators_.
//   - bufOffset_ always points to the location of the start of the next token in buf_,
//      or to the null terminator in buf_.

class Lex {
  public:
    Lex(const char* str, const char* separators=0, const char* operators=0);
    ~Lex() {}

    char* token();              // return next token as pointer to local storage, or null

    void lexString(const char* str);  // set a new string to analyze using current seps and ops
    void setSeparators(const char* separators, const char* operators);  // set new seps and ops
    void restoreSeparators();   // restore seps and ops to ctor values

  private:
    // some class constants
    enum { TOKENBUFSIZE = 1024, TERMBUFSIZE = 64 };
    static char* NULLSTRING;    // the null string

    void initTerminators();     // init fn
    void initBuf();             // init fn

    bool atOperator() const;    // is next token an operator?
    void eatSeparators();       // advance to next non-separator char in buf_
    bool endTest() const 
    { return buf_[bufOffset_] == 0; } // end of line has been reached?

    const char* buf_;           // string being analyzed
    char token_[TOKENBUFSIZE];  // token to return
    const char* separators_;    // separating characters
    const char* operators_;     // single character tokens
    char terminators_[TERMBUFSIZE]; // concatenation of separators_ and operators_

    const char* ctorSeparators_; // separators as given in the ctor
    const char* ctorOperators_;  // operators as given in the ctor

    int bufOffset_;             // current offset in buf_
    int tokenOffset_;           // current offset in token_ under construction
};

#endif // Lex_H
