/**********************************************************************
  Columbia Basin Research Software Library Tools
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#ifndef LineBuffer_H
#define LineBuffer_H

// LineBuffer class.  Opens a file using ifstream and provides
// functions for line by line processing.  nextLine() returns null
// when the end of the file has been reached.  The token fn
// nextLineToken() returns the
// next token on the current line, and returns null when there are no
// remaining tokens on the current line.  Tokens are defined as
// whitespace delimited strings; the token fn automatically skips
// intervening whitespace.  This class will throw ParseError
// exceptions if unable to open the requested fstream, or on a bad
// read.

#include <fstream.h>
#include <rw/cstring.h>
#include "Lex.h"

class LineBuffer {
  public:
    enum { LineSize=1024 };
    
    LineBuffer(const RWCString& fileName, const char* separators=0,
		                 const char* operators=0);
    ~LineBuffer() {}
    
    RWCString fileName() const { return fileName_; }
    long lineCount() const { return lineCount_; }

    // read next line, or return current one
    const char *nextLine();
    const char *currentLine() const
    { return buf_; }

    // get first/next token in line.
    RWCString nextLineToken();

    // change separators in the tokenizer
    void setSeparators(const char* separators, const char* operators)
    { tokenizer_.setSeparators(separators, operators); }
    void restoreSeparators()
    { tokenizer_.restoreSeparators(); }
     
  protected:
    // this ref to the istream only good while this object is extant
    const ifstream& ifs() const { return ifs_; }

  private:
    const RWCString fileName_;
    ifstream ifs_;
    long lineCount_;
    char buf_[LineSize+1];

    Lex tokenizer_;

    // make new tokenizer for buf_, get nextToken_.
    void updateTokenizer();
    
    // clear tokenizer, clear tokens.
    void clearTokenizer();
};

#endif // LineBuffer_H
