/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "MaturationManager.h"
#include "Maturation.h"
#include "Stocks.h"
#include "Chronograph.h"
#include "Geographer.h"
#include "GenericDefaultArray.h"
#include "SystemClock.h"

class Cohort;

const RCPtr<Maturation> MaturationManager::defaultMaturation_ = new Maturation(0.0);
auto_ptr<MatTable> MaturationManager::matTable_ 
  = auto_ptr<MatTable>(new GenericDefaultArray<RCPtr<Maturation> >(defaultMaturation_));

// reconfigure table to default values
void MaturationManager::
config()
{
    matTable_ = auto_ptr<MatTable>(new GenericDefaultArray<RCPtr<Maturation> >(defaultMaturation_));
}

// reconfig with a new table; this object assumes ownership of the new maturation table supplied here.
void MaturationManager::
config(auto_ptr<MatTable> newTable)
{
    matTable_ = newTable;	// auto_ptr deletes any old table
}

void
MaturationManager::insertMaturation(const RCPtr<Maturation>& mat,
				    const GenericArrayIndex& index)
{
    (*matTable_)[index] = mat;
}

double MaturationManager::
maturationRate(const GenericArrayIndex& index)
{ 
    return ((*matTable_)[index])->maturationRate(); 
}

// loop through cohorts and regions, extract the proper Maturation
// object, and call its maturate() method.
void
MaturationManager::maturateCohorts()
{
    for (int i = 0; i < Stocks.num(); ++i) {
        CohortIter next(Stocks[i].getCohorts());
        Cohort* cohort;
        while (cohort = next()) {
            for (unsigned locIndex=0; locIndex < Geographer::nRegions(); ++locIndex) {
                GenericArrayIndex id(cohort, 0, locIndex, ::SystemClock->currentYearIndex(),
                    ::SystemClock->currentTimeIndex());
                const MatTable& table = *matTable_;
                table[id]->maturate(cohort, id);
            }
        }
    }
}
