/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "PSCShaker.h"
#include "Fishery.h"
#include "HvMort.h"
#include "Cohort.h"
#include "Stocks.h"
#include "GenericArray.h"
#include "HarvestManager.h"

#include <rw/tpordvec.h>
#include <rw/math/mathvec.h>


// shaker mortality in the current timestep.  also sets the encounter rate in the fishery.

void PSCShaker::
setShakers(Fishery* fishery) const
{
    double wgtd_pv = 0;
    double wgtd_pnv = 0;
    double encounter_rate = 0;
    
    RWMathVec<double> stockcatch(Stocks.num(), 0.0);
    RWMathVec<double>& stockweight = stockcatch;
    
    const RWTPtrOrderedVector<HvMort>& harvests = fishery->harvests();
    
    for (int i=0; i < harvests.length(); ++i) {
        stockcatch(harvests(i)->cohort().stock()->index()) += harvests(i)->legalCatch();
    }
    double f_catch = sum(stockcatch);
    
    // set the encounter rate (and possibly pv/pnv weights)
    if (f_catch != 0) {
        // weight the stock catch
        stockweight /= f_catch;
        for (i=0; i < harvests.length(); ++i) {
            // set weighted pnvs
            if (vulnerable(harvests(i))) {
                double wgtd_cohort =
                    stockweight(harvests(i)->cohort().stock()->index())
                    * harvests(i)->cohortAbundance();
                double apnv = HarvestManager::pnv(harvests(i));
                wgtd_pnv += wgtd_cohort * apnv;
                wgtd_pv  += wgtd_cohort * (1.0 - apnv);
            }
        }
        encounter_rate = (wgtd_pv) ? wgtd_pnv / wgtd_pv : 0;
    }
    
    double shakers = f_catch * encounter_rate * (subLegalShakerMortRate() + dropOffMortRate());
    
    // set shaker morts in harvests
    for (i=0; i < harvests.length(); ++i) {
        if (vulnerable(harvests(i))) {
            // this test catches wgtd_pnv divide by zero error 
            if (shakers > 0.0) {
                double wgtd_cohort =
                    stockweight(harvests(i)->cohort().stock()->index())
                    * harvests(i)->cohortAbundance();
                double frac_nv =
                    wgtd_cohort * HarvestManager::pnv(harvests(i)) / wgtd_pnv;
                harvests(i)->shakers() = frac_nv * shakers;
                harvests(i)->dropOffs() = dropOffs(harvests(i));
            }
            else {
                harvests(i)->shakers() = 0.0;
                harvests(i)->dropOffs() = 0.0;
            }
        }
    }
    fishery->encounterRate(encounter_rate);
}

