/**********************************************************************
  Columbia Basin Research Software Library Tools
  Copyright (C) 1998,1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// Parser implementations

#include "rwstrstream.h"
#include <rw/dvec.h>
#include "Parser.h"

// Constructors may or may not set up custom error logging object.
Parser::Parser(const RWCString& fileName,
			   const char* separators, const char* operators)
 : FreeFormReader(fileName, separators, operators)
{
    customErrorLogger = rwnil;
}

Parser::Parser(const RWCString& fileName, ParserErrorLogger *pel,
			   const char* separators, const char* operators)
        : FreeFormReader(fileName, separators, operators)
{
    customErrorLogger = pel;
}

// default parsing fn.  virtual so that it may be overridden in the
// derived class if desired.  this fn churns through the whole file,
// dumping error messages as they appear.  ParseError exceptions are
// caught and output.

void
Parser::parse()
{
    RWCString token;
    while (!((token = nextToken()).isNull())) {
        try {
            processKeyword(token);
        }
        catch (ParseError &err) {
            logError(err.errorMessage());
        }
    }
}

// log an error message.  this will eventually make use of a contained
// object, configured by the user, to decide how to do the error
// logging.  for now, just send it to cerr.

#include <iostream.h>

void
Parser::logError(const RWCString& msg) const
{
    if (customErrorLogger == rwnil)
        cerr << msg << endl;
    else
        customErrorLogger->logError( msg );
}

// basic data routines.  get the next token and parse it into a var,
// placing the result in the passed reference parameter.

void Parser::
parseVar(RWCString& var)
{
    // strip surrounding quotes
    var = nextToken().strip(RWCString::both, '"');
}

void Parser::
parseVar(int& var)
{
    RWCString token = nextToken();
    istrstream s(const_cast<char *>(token.data()));
    if (!(s >> var))
    	throw pError(ParseError::BadType, "integer expected", token);
    char c;
    if (s >> c)
        throw pError(ParseError::BadType, "single integer expected", token);
}

void Parser::
parseVar(long& var)
{
    RWCString token = nextToken();
    istrstream s(const_cast<char *>(token.data()));
    if (!(s >> var))
    	throw pError(ParseError::BadType, "long expected", token);
    char c;
    if (s >> c)
        throw pError(ParseError::BadType, "single long integer expected", token);
}

void Parser::
parseVar(float& var)
{
    RWCString token = nextToken();
    istrstream s(const_cast<char *>(token.data()));
    if (!(s >> var))
    	throw pError(ParseError::BadType, "float expected", token);
    char c;
    if (s >> c)
        throw pError(ParseError::BadType, "single float expected", token);
}

void Parser::
parseVar(double& var)
{
    RWCString token = nextToken();
    istrstream s(const_cast<char *>(token.data()));
    if (!(s >> var))
    	throw pError(ParseError::BadType, "double expected", token);
    char c;
    if (s >> c)
        throw pError(ParseError::BadType, "single double expected", token);
}

// read a range of integers.  format is "x:y" without whitespace.
void Parser::
parseVar(Range<int>& var)
{
    RWCString token = nextToken();
    istrstream s1(const_cast<char *>(token.data()));
    if (!(s1 >> var.first))
    	throw pError(ParseError::BadType, "integer range x:y expected", token);

    token = nextToken();
    if (token != ":")
    	throw pError(ParseError::BadType, "integer range x:y expected", token);

    token = nextToken();
    istrstream s2(const_cast<char *>(token.data()));
    if (!(s2 >> var.last))
    	throw pError(ParseError::BadType, "integer range x:y expected", token);
}


#ifdef RWMATH		       

// Parse a rogue wave DoubleVec. "dvec" has an incoming length - allow 
//  fewer values than that if an early-cutoff token ";" is seen. Allow 
//  run-coding of the form:  "[" <index> ":" <index> "]" <value> 
//           or of the form: "[" "*" "]" <value>
//  or single values. Non-specified values are zeroed. The tokenizer
//  is set to recognize "[];:*" as opeators (special one-character
//  tokens).

void Parser::parseVar(DoubleVec& dvec)
{
    int vecLen = dvec.length();
    int i, ind1, ind2;
    int index = 0;
    RWCString token, nextTok;
    double d;

    // Zero out the entire vector.
    dvec = 0.0;

    // Read tokens until the vector is full or ";" is seen.
    while (index < vecLen) {
		token = nextToken();

		// Are we done yet?
		if (token(0) == ';')
			break;

		// Do we detect the beginning of a run-coded range?
		if (token(0) == '[') {
			nextTok = peekToken();
			if (nextTok(0) == '*') {
				// "[*" detected - range is entire vector, 
				//  look for trailing "]"
				token = nextToken();  // discard '*'
				ind1 = 0;
				ind2 = vecLen-1;
			}
			else {
				// Get first integer that defines range.
				parseVar( ind1 );

				// Look for ":" expected.
				token = nextToken( );
				if (token(0) != ':')
					throw pError( ParseError::BadType,
									 ": expected", token );

				// Get second integer that defines range.
				parseVar( ind2 );
			}

			// Look for trailing bracket of "[*]" or "[n:n]"
			token = nextToken( );
			if (token(0) != ']')
				throw pError( ParseError::BadType, "] expected", token );

			// Get double value which must come next.
			parseVar( d );

			// Make sure range is reasonable.
			if (ind1 < 0 || ind2 >= vecLen || ind1 > ind2)
				throw pError( ParseError::BadData,
									"vector length mismatch", token );

			// Fill indicated range of vector - set input pointer to element
			//  which follows that range.
			for (i = ind1; i <= ind2; i++)
				dvec(i) = d;
			index = ind2 + 1;
		}
        else {
            // Normal processing of a numeric value.
			istrstream s(const_cast<char *>(token.data()));
			if (!(s >> d))
			throw pError(ParseError::BadType, "double expected", token);
			char c;
			if (s >> c)
				throw pError(ParseError::BadType, "single double expected", token);
			dvec(index++) = d;
        }
    }
}

#endif // RWMATH
