/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "PnvParser.h"
#include "HarvestTokens.h"

#include "HarvestManager.h"
#include "GenericArrayFactory.h"


// ctor utility fns
void
PnvParser::initKeymap()
{
    keymap_.insert(HarvestTokens::PNV, pnv);
    initGenArrayKeymap(keymap_);
}

void
PnvParser::initState()
{
    // configure the initial state with the correct id list.
    State* state = new State;
    state->context = "PnvData";
    configPnv(state);
    stateStack.push(state);
}

PnvParser::PnvParser(Parser& p, RWCString dimension)
    : GenArrayParser(p), dimension_(dimension)
{
    initState();
    initKeymap();
}

// create a generic array for pnv data of the requested dimension size
// and configure the HarvestManager with it.  also add the list of
// generic ids to index every element of the array to the current
// state.

void
PnvParser::configPnv(State* state)
{
    GenericArrayFactory<double> pnvFactory;
    GenericArrayFactory<double>::GenericArrayType arraytype = pnvFactory.genericArrayType(dimension_); 
    if (arraytype != pnvFactory.NTypes) {
        PnvTable* pnvTable = pnvFactory.newGenericArrayFn1(arraytype, HarvestManager::defaultPnv_); 
        HarvestManager::configPnvs(auto_ptr<PnvTable>(pnvTable)); // hand off harvest storage to the manager
        pnvTable->newGenericIndexList(state->ids); // create the proper list of generic ids
    }
    else {
        // unidentified dimension.  log an error, but allow this
        // parser to continue with an empty id list in the state, so
        // as to skip as much data as possible without generating
        // further messages.
        logError(pError(ParseError::BadData,
            dimension_ + " is not a valid configuration for"
            + state->context + "; discarding data")
            .errorMessage());
    }
}


// read in one pnv object and place it in the HarvestManager
// at every generic id region in the current list
void
PnvParser::pnv()
{
    double pnv;
    parseVar(pnv);

    // iterate through the ids in the current state
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
		// hand off the ptr to the migration manager
		HarvestManager::insertPnv(pnv, *id);
        ++iter;
    }
    // pop one level of state after processing the data
    parseEnd();
}
