/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "ProductionParser.h"
#include "ProductionTokens.h"

#include "LinearProduction.h"
#include "RickerProduction.h"
#include "EnhancedRickerProduction.h"
#include "VariableTruncationRickerProduction.h"
#include "Production.h"
#include "SpawningManager.h"
#include "GenericArrayFactory.h"


// ctor utility fns
void
ProductionParser::initKeymap()
{
    keymap_.insert(ProductionTokens::PRODUCTION, production);
    initGenArrayKeymap(keymap_);
}

void
ProductionParser::initState()
{
    // configure the initial state with the correct id list.
    State* state = new State;
    state->context = "ProductionFunctions";
    configProductionTable(state);
    stateStack.push(state);
}

ProductionParser::ProductionParser(Parser& p, RWCString dimension)
    : GenArrayParser(p), dimension_(dimension)
{
    initState();
    initKeymap();
}

// create a generic array for production data of the requested
// dimension size and configure the SpawningManager with it.  also add
// the list of generic ids to index every element of the array to the
// current state.

void
ProductionParser::configProductionTable(State* state)
{
    GenericArrayFactory<auto_ptr<Production> > prodFactory;
    GenericArrayFactory<auto_ptr<Production> >::GenericArrayType arraytype = prodFactory.genericArrayType(dimension_); 
    if (arraytype != prodFactory.NTypes) {
        ProductionTable* prodTable = prodFactory.newGenericArrayFn0(arraytype);  
        prodTable->newGenericIndexList(state->ids); // create the proper list of generic ids
        // permanently hand off production storage to the manager
        SpawningManager::configProduction(auto_ptr<ProductionTable>(prodTable)); 
    }
    else {
        // unidentified dimension.  log an error, but allow this
        // parser to continue with an empty id list in the state, so
        // as to skip as much data as possible without generating
        // further messages.
        logError(pError(ParseError::BadData,
            dimension_ + " is not a valid configuration for"
            + state->context + "; discarding data")
            .errorMessage());
    }
}


// read in one ProductionFunction object and place it in the SpawningManager
// at every generic id region in the current list
void
ProductionParser::production()
{
    RWCString prodFnName;
    parseVar(prodFnName);
    prodFnName.toLower();

    auto_ptr<const ProductionFunction> productionFn;
    if (prodFnName == "linear") {
        double minSpawners, maxSpawners, ev, slope;
        parseVar(minSpawners);
        parseVar(maxSpawners);
        parseVar(ev);
        parseVar(slope);
        productionFn.reset(new LinearProduction(minSpawners, maxSpawners, ev, slope));
    }
    else if (prodFnName == "ricker") {
        double minSpawners, maxSpawners, ev, rickerA, rickerB, age1recruits;
        parseVar(minSpawners);
        parseVar(maxSpawners);
        parseVar(ev);
        parseVar(rickerA);
        parseVar(rickerB);
        parseVar(age1recruits);
        productionFn.reset(new RickerProduction(minSpawners, maxSpawners, ev, rickerA, 
            rickerB, age1recruits));
    }
    else if (prodFnName == "enhancedricker") {
        double minSpawners, maxSpawners, ev, rickerA, rickerB, age1recruits;
        int densityDependence;
        double productivityParam, smoltAtAge1, maxBroodProportion, smoltProductionChange;

        parseVar(minSpawners);
        parseVar(maxSpawners);
        parseVar(ev);
        parseVar(rickerA);
        parseVar(rickerB);
        parseVar(age1recruits);
        parseVar(densityDependence);
        bool densityDepFlag = (densityDependence==0) ? false : true;
        parseVar(productivityParam);
        parseVar(smoltAtAge1);
        parseVar(maxBroodProportion);
        parseVar(smoltProductionChange);
        productionFn.reset(new EnhancedRickerProduction(minSpawners, maxSpawners, ev, rickerA, rickerB, 
            age1recruits, densityDepFlag, productivityParam, smoltAtAge1, maxBroodProportion,
            smoltProductionChange));
    }
	else if (prodFnName == "variabletruncationricker") {
        double minSpawners, maxSpawners, ev, rickerA, rickerB, age1recruits;
        parseVar(minSpawners);
        parseVar(maxSpawners);
        parseVar(ev);
        parseVar(rickerA);
        parseVar(rickerB);
        parseVar(age1recruits);
        productionFn.reset(new VariableTruncationRickerProduction(minSpawners, maxSpawners, ev, rickerA, 
            rickerB, age1recruits));
    }
    else {
		logError(pError(ParseError::BadData,
            prodFnName + " is not a valid production function; "
            + stateStack.top()->context + "; discarding data")
            .errorMessage());
		nextDataLine();		// skip the rest of this line 	
		return;
	}
    // iterate through the ids in the current state
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        // make a new auto_ptr for each productionFn by cloning, and insert into SpawningManager
        SpawningManager::insertProductionFunction(
            auto_ptr<const ProductionFunction>(productionFn->clone()), *id);
        ++iter;
    }
    // multiple production fns are allowed, so don't pop any state (i.e. no parseEnd() )
}
