/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// SDS_Fmrt.cpp - Standard Data Sentence Fishing Mortality

#include "SDS_Fmrt.h"
#include "Chronograph.h"
#include "Cohort.h"
#include "SystemClock.h"
#include "Geographer.h"
#include "HvMort.h"
#include "GenericArray.h"
#include "FisheryManager.h"
#include "Fisheries.h"
#include "StandardDataSentenceState.h"

// unique identifier for this sentence type
const char* SDS_Fmrt::fmrtId_ = "FMRT";

SDS_Fmrt::SDS_Fmrt(ofstream& outfile) : StandardDataSentence(fmrtId_, outfile)
{}


void SDS_Fmrt::collect(SystemProcesses::ProcessId pid)
{
    // output fishing mortalities after harvest timestep
    if (pid == SystemProcesses::SP_Harvest) {
        for (unsigned f=0; f < ::Fisheries.num(); ++f) {
            const Fishery* fishery = &::Fisheries[f];
            for (unsigned h=0; h < fishery->harvests().entries(); ++h) {
                const HvMort* hvmort = (fishery->harvests())(h);
                // ouput a sentence with any nonzero data
                if (hvmort->legalCatch() != 0.0
                    || hvmort->shakers() != 0.0
                    || hvmort->dropOffs() != 0.0
                    || hvmort->cnrTotal() != 0.0
                    ) {
                        GenericArrayIndex gendex(&(hvmort->cohort()), fishery->index(), 
                            hvmort->region().index(), *::SystemClock);
                        double effort = FisheryManager::effortScalar(gendex);
                        static char buf[1024];
/*                        ostrstream s(buf, 1024);
                        s << Chronograph::currentCalendarYear()
                          << ", " << ::SystemClock->currentTimeIndex()+1  // +1 converts to user time
                          << ", " << hvmort->region().index()+1 // +1 converts to user regions
                          << ", " << fishery->number() // user fishery number
                          << ", " << hvmort->cohort().id().idString()
                          << ", " << hvmort->legalCatch()
                          << ", " << hvmort->shakers()
                          << ", " << hvmort->dropOffs()
                          << ", " << hvmort->cnrSubLegal()
                          << ", " << hvmort->cnrLegal()
                          << ", " << effort
                          << '\n';
                        buf[s.pcount()] = 0;
*/
                        sprintf(buf, "%d, %d, %d, %d, %s, %.6g, %.6g, %.6g, %.6g, %.6g, %.6g\n",
                            Chronograph::currentCalendarYear(),
                            ::SystemClock->currentTimeIndex()+1,  // +1 converts to user time
                            hvmort->region().index()+1, // +1 converts to user regions
                            fishery->number(), // user fishery number
                            hvmort->cohort().id().idString(),
                            hvmort->legalCatch(),
                            hvmort->shakers(),
                            hvmort->dropOffs(),
                            hvmort->cnrSubLegal(),
                            hvmort->cnrLegal(),
                            effort
                            );

                        outputSentence(buf);
                }
            }
        }
        outfile_.flush();
    }
}


State* SDS_Fmrt::
newSaveState()
{
    return (new StandardDataSentenceState(outfile_)); 
}
