/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "ShakerParser.h"
#include "ShakerTokens.h"

#include "PSCShaker.h"
#include "PSCCustomShaker.h"
#include "PSCDropShaker.h"
#include "PSCCustomDropShaker.h"
#include "HarvestManager.h"
#include "VulnerabilityTableParser.h"
#include "GenericArrayFactory.h"
#include "GenericDefaultArray.h"


// ctor utility fns
void
ShakerParser::initKeymap()
{
    keymap_.insert(ShakerTokens::METHOD, method);
    keymap_.insert(ShakerTokens::PSCSIMPLE, simpleMethod);
    keymap_.insert(ShakerTokens::PSCCUSTOM, customMethod);
    keymap_.insert(ShakerTokens::PSCSIMPLEDROP, simpleDropMethod);
    keymap_.insert(ShakerTokens::PSCCUSTOMDROP, customDropMethod);
    keymap_.insert(ShakerTokens::SHAKERMORTRATE, subLegalReleaseMortRate);
    keymap_.insert(ShakerTokens::SUBLEGALRELEASEMORTRATE, subLegalReleaseMortRate);
    keymap_.insert(ShakerTokens::DROPOFFRATE, dropOffRate);
    keymap_.insert(ShakerTokens::VULNERABILITYTABLE, vulnerabilityTable);
    initGenArrayKeymap(keymap_);
}

void
ShakerParser::initState()
{
    // configure the initial state with the correct id list.
    State* state = new State;
    state->context = "ShakerData";
    configShaker(state);
    stateStack.push(state);
}

ShakerParser::ShakerParser(Parser& p, RWCString dimension)
    : GenArrayParser(p), dimension_(dimension),
      vulnerabilityTable_(auto_ptr<GenericArray<bool> >(new GenericDefaultArray<bool>(false))),
      vtabInit_(false)
{
    initState();
    initKeymap();
    clearData();
}

void ShakerParser::clearData()
{
    shakerType_ = ShakerNone;
    subLegalReleaseMortRate_ = 0.0;

    // reference counting causes the old table to leave scope
    vulnerabilityTable_ = auto_ptr<GenericArray<bool> >(new GenericDefaultArray<bool>(false));
    vtabInit_ = false;
}

// create a generic array for shaker data of the requested dimension size
// and configure the HarvestManager with it.  also add the list of
// generic ids to index every element of the array to the current
// state.

void
ShakerParser::configShaker(State* state)
{
    GenericArrayFactory<auto_ptr<Shaker> > shakerFactory;
    GenericArrayFactory<auto_ptr<Shaker> >::GenericArrayType arraytype = shakerFactory.genericArrayType(dimension_); 
    if (arraytype != shakerFactory.NTypes) {
        // table entries are null auto_ptrs by default
        ShakerTable* shakerTable = shakerFactory.newGenericArrayFn0(arraytype); 
        HarvestManager::configShakers(auto_ptr<ShakerTable>(shakerTable)); // hand off harvest storage to the manager
        shakerTable->newGenericIndexList(state->ids); // create the proper list of generic ids
    }
    else {
        // unidentified dimension.  log an error, but allow this
        // parser to continue with an empty id list in the state, so
        // as to skip as much data as possible without generating
        // further messages.
        logError(pError(ParseError::BadData,
            dimension_ + " is not a valid configuration for"
            + state->context + "; discarding data")
            .errorMessage());
    }
}

void ShakerParser::parseEnd()
{
    // if a shaker type exists, then we are ending a method definition, 
    // in which case we create all the new shaker objects, parse the end token,
    // and clear the data (which also nulls out the shaker type).  
    // otherwise, do the standard GenArray end parse, which 
    // pops a level of state.
    
    if (shakerType_ != ShakerNone) {
        // iterate through the ids in the current state
        RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
        bool err = false;
        while (iter != (stateStack.top()->ids).end() && !err) {
            GenericArrayIndex* id = *iter;
    		// make object, hand off the ptr to the harvest manager, which assumes ownership
            Shaker* shaker = 0;
            switch (shakerType_) {
            case ShakerSimple:
                shaker = new PSCShaker(subLegalReleaseMortRate_);
                break;

            case ShakerCustom:
                if (vtabInit_)
                    shaker = new PSCCustomShaker(subLegalReleaseMortRate_, vulnerabilityTable_);
                else {
                    logError(pError(ParseError::BadData,
                        "no vulnerability table specified!  discarding; "
                        + context()).errorMessage());
                    err = true;
                }
                break;

            case ShakerSimpleDrop:
                shaker = new PSCDropShaker(subLegalReleaseMortRate_, dropOffRate_);
                break;

            case ShakerCustomDrop:
                if (vtabInit_)
                    shaker = new PSCCustomDropShaker(subLegalReleaseMortRate_, dropOffRate_, vulnerabilityTable_);
                else { 
                    logError(pError(ParseError::BadData,
                        "no vulnerability table specified!  discarding; "
                        + context()).errorMessage());
                    err = true;
                }
                break;
            }
		    HarvestManager::insertShaker(auto_ptr<Shaker>(shaker), *id);
            ++iter;
        }
        clearData();        // this also resets the shakerType_
        nextDataLine();		// skip the rest of this line as comments
    }
    else
        GenArrayParser::parseEnd();
}

// begin parsing a method definition
void ShakerParser::method()
{
    clearData();
}

// choose methods
void ShakerParser::simpleMethod()
{ 
    shakerType_ = ShakerSimple;
}
void ShakerParser::customMethod()
{
    shakerType_ = ShakerCustom;
}
void ShakerParser::simpleDropMethod()
{ 
    shakerType_ = ShakerSimpleDrop;
}
void ShakerParser::customDropMethod()
{
    shakerType_ = ShakerCustomDrop;
}

void
ShakerParser::subLegalReleaseMortRate()
{
    parseVar(subLegalReleaseMortRate_);
}

void ShakerParser::dropOffRate()
{
    parseVar(dropOffRate_);
}

void ShakerParser::vulnerabilityTable()
{
    RWCString dimension = nextToken();
    VulnerabilityTableParser vtabp(*this, dimension);
    vtabp.parse();
    vulnerabilityTable_ = vtabp.getTable(); // get a ref-counted copy of the table
    vtabInit_ = true; // flag the vulnerability table as initialized
}