/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// stock class declarations

#ifndef STOCK_H
#define STOCK_H

#include "pragma.h"

#include <assert.h>
#include "Age.h"
#include "CohortList.h"
#include "Geographer.h"
#include "Chronograph.h"
#include "RegionList.h"
#include "CohortGenerator.h"

#include <rw/tpordvec.h>
#include <rw/tvordvec.h>
#include <rw/cstring.h>

class Fishery;
class HvMort;
class Production;

class Stock {
  public:
    Stock(const RWCString& nm);
    // at the moment stocks can't be copied - this fn satisfies RW classes
    Stock(const Stock&) { assert(0); }
    ~Stock() {}
    
    // interface to data
    const char* get_name() const { return name; }
    const char* get_abbrv() const { return abbrv; }
    unsigned index() const { return index_; }
    unsigned number() const { return number_; }
    CohortID::ProductionType prodType() const { return prodType_; }
    CohortID::RunType run() const { return run_; }

    Age maxAge() const { return maxAge_; }
    Age firstHarvestAge() const { return firstHarvestAge_; }
    Age adultAge() const { return adultAge_; }

    double adultMatureRegionalAbundance(unsigned regionIndex) const;
    const CohortList& getCohorts() const { return cohorts_; }
    CohortList& getCohorts() { return cohorts_; }
    Cohort* findActiveCohort(const CohortID& id);
    
    // actions
    void config();		// data inits after reading most data
    void activateCohort(auto_ptr<Cohort> cohort); // add a new cohort to the active list and assume ownership
    void ageCohorts();      // age all active cohorts
    void destroyCohorts()
    { cohorts_.clearAndDestroy(); }

    // spawn() returns new age0 abundance for one region.
    double spawn(const Production&, unsigned regionIndex); // new production

    void newAge0Cohorts(double abd, unsigned region, unsigned broodyear) // newly produced cohorts are registered
    { cohortGenerator_->newAge0Cohorts(abd, region, this, broodyear); }

    // operators
    // op= and op== for Rogue Wave
    Stock& operator=(const Stock &)
	{ assert(0); return *this; } // disabled
    int operator==(const Stock &rhs) const
	{ return (&rhs==this); }
    
private:
    friend class StockConfigParser;

    void index(unsigned index) { index_ = index; } // this MUST be set for proper operation

    const RWCString name;
    const RWCString abbrv;

    unsigned index_;			// unique ID suitable for array indexing
    unsigned long number_;      // externally assigned number
    CohortID::ProductionType prodType_; // wild, hatchery, etc.
    CohortID::RunType run_;     // fall, spring, etc.
    // various age characteristics for this stock
    Age firstHarvestAge_;
    Age adultAge_;
    Age maxAge_;
    
    auto_ptr<CohortGenerator> cohortGenerator_; // creation and distribution of new cohorts
    
    DestructiveCohortList cohorts_;    // this stock object owns these cohort pointers
};


#endif
