/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// SystemState.h - class to hold a snapshot of the system state

#ifndef SystemState_H
#define SystemState_H

#include "State.h"
#include "RCPtr.h"
#include "RCObject.h"
#include <rw/tpslist.h>

// this class holds a snapshot of the state of the system.  it only deals with
// the portions of the state which are variable.  the fixed state of the system is
// handled by external global variables, the various data "Managers".  also, 
// "control" variables used by iteration objects are not considered to be part of 
// the system state.  the fundamental control variable is the PolicyTable.
// since this class manages a large amount of data, it is reference-counted for 
// space efficiency.  this class is implemented using the Composite pattern.  it
// simply maintains a list of other State objects which handle the variable state
// for various portions of the system.

class SystemState : public State {
  public:
    SystemState();
    virtual ~SystemState();

    virtual void restoreState() const;      // restore system state to stored value

  protected:
    virtual void saveState();               // take a snapshot of current system state

  private:
    // the data representation is reference counted
    struct StateRep : private RCObject {
        friend class RCPtr<StateRep>;
        StateRep() {}
        ~StateRep() { stateComponents_.clearAndDestroy(); }

        // the RCPtrs may be copied, but not this RCObject.  this is a bit of defensive
        // programming illustrating that point - there are no definitions for these initializers
        StateRep(const StateRep&);
        StateRep& operator=(const StateRep&);

        // the data
        RWTPtrSlist<State> stateComponents_;
    };
    // the RCPtr copy ctor automagically reference counts the stateRep_ when a SystemState is copied.
    // like auto_ptr, the underlying heap memory is also deleted when appropriate
    RCPtr<StateRep> stateRep_;
};

#endif // SystemState_h