/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "TopLevelParser.h"
#include "Tokens.h"
#include "TopTokens.h"
#include "TransMatParser.h"
#include "MaturationParser.h"
#include "HarvestParser.h"
#include "FpParser.h"
#include "NaturalMortalityParser.h"
#include "PnvParser.h"
#include "ShakerParser.h"
#include "CNRParser.h"
#include "ConfigParser.h"
#include "CeilingScalarParser.h"
#include "CeilingParser.h"
#include "CohortConfigParser.h"
#include "ProductionParser.h"
#include "FisheryScheduleParser.h"
#include "ErrorLogObject.h"

TopLevelParser::
TopLevelParser(const RWCString& fileName)
: Parser(fileName, &::errorLogObject)
{
    keymap_.insert(Tokens::INCLUDE, parseInclude);
    keymap_.insert(TopTokens::CONFIGURATION, config);
    keymap_.insert(TopTokens::TRANSMAT, transitionMatrix);
    keymap_.insert(TopTokens::MATURATION, maturationData);
    keymap_.insert(TopTokens::HVRATE, harvestData);
    keymap_.insert(TopTokens::FPDATA, fpData);
    keymap_.insert(TopTokens::NATMORTDATA, natMortData);
    keymap_.insert(TopTokens::PNVDATA, pnvData);
    keymap_.insert(TopTokens::SHAKERDATA, shakerData);
    keymap_.insert(TopTokens::CNRDATA, cnrData);
    keymap_.insert(TopTokens::CEILINGSCALAR, ceilingScalars);
    keymap_.insert(TopTokens::MULTICEILINGSCALAR, multiCeilingScalars);
    keymap_.insert(TopTokens::CEILINGDATA, ceilingData);
    keymap_.insert(TopTokens::COHORTS, cohortData);
    keymap_.insert(TopTokens::PRODUCTIONFNS, productionData);
    keymap_.insert(TopTokens::FISHERYSCHEDULE,  fisherySchedule);
}

void
TopLevelParser::processKeyword(const RWCString& token)
{
    keymap_.processKeyword(*this, token);
}

void
TopLevelParser::parseInclude()
{
    
    RWCString filename = nextToken();
    try {
        TopLevelParser p(filename);
        p.parse();
    }
    catch(ParseError& err) {
        // modify the error message if it was BadFile (add current
        // filename and line number)
        if (err == ParseError::BadFile)
            throw pError(ParseError::BadFile,
            "unable to open file \"" + filename + "\""
            );
        else throw;
    }
}

void
TopLevelParser::config()
{
    ConfigParser config(*this);
    config.parse();
}

void
TopLevelParser::transitionMatrix()
{
    RWCString dimension = nextToken();
    TransMatParser tm(*this, dimension);
    tm.parse();
}

void
TopLevelParser::maturationData()
{
    RWCString dimension = nextToken();
    MaturationParser mp(*this, dimension);
    mp.parse();
}

void
TopLevelParser::harvestData()
{
    RWCString dimension = nextToken();
    HarvestParser hp(*this, dimension);
    hp.parse();
}

void
TopLevelParser::fpData()
{
    RWCString dimension = nextToken();
    FpParser fp(*this, dimension);
    fp.parse();
}

void
TopLevelParser::natMortData()
{
    RWCString dimension = nextToken();
    NaturalMortalityParser nmp(*this, dimension);
    nmp.parse();
}

void
TopLevelParser::pnvData()
{
    RWCString dimension = nextToken();
    PnvParser pnvp(*this, dimension);
    pnvp.parse();
}

void
TopLevelParser::ceilingScalars()
{
    CeilingScalarParser ceilingp(*this, false);  // the bool indicates no multi ceiling
    ceilingp.parse();
}

void
TopLevelParser::multiCeilingScalars()
{
    CeilingScalarParser ceilingp(*this, true);  // multi ceiling flag thrown
    ceilingp.parse();
}

void
TopLevelParser::ceilingData()
{
    CeilingParser ceilingp(*this);
    ceilingp.parse();
}

void 
TopLevelParser::cohortData()
{
    CohortConfigParser cohortp(*this);
    cohortp.parse();
}

void 
TopLevelParser::productionData()
{
    RWCString dimension = nextToken();
    ProductionParser productionp(*this, dimension);
    productionp.parse();
}

void
TopLevelParser::shakerData()
{
    RWCString dimension = nextToken();
    ShakerParser shakerp(*this, dimension);
    shakerp.parse();
}

void
TopLevelParser::cnrData()
{
    RWCString dimension = nextToken();
    CNRParser cnrp(*this, dimension);
    cnrp.parse();
}

void 
TopLevelParser::fisherySchedule()
{
    FisheryScheduleParser fsparser(*this);
    fsparser.parse();
}