/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// TrackDetailedFisheryCatches.cpp

#include "Geographer.h"
#include "Chronograph.h"
#include "SystemClock.h"
#include "TrackDetailedFisheryCatches.h"
#include "TrackDetailedFisheryCatchState.h"
#include "Fisheries.h"

// copy ctor makes a deep copy
TrackDetailedFisheryCatches::TrackDetailedFisheryCatches(const TrackDetailedFisheryCatches& rhs)
{   
    // use the assignment operator for RWGenMat to get a deep copy
    config();
    catches_ = rhs.catches_;
}

// assignment operator also makes a deep copy
TrackDetailedFisheryCatches& TrackDetailedFisheryCatches::
operator=(const TrackDetailedFisheryCatches& rhs)
{
    if (this != &rhs) {
        config();
        catches_ = rhs.catches_;
    }
    return *this;
}

void TrackDetailedFisheryCatches::
config()
{
    catches_.reshape(::Fisheries.entries(), Geographer::nRegions(), 
        Chronograph::nYears(), Chronograph::nTimeSteps());
    clear();
}


void TrackDetailedFisheryCatches::
collect(SystemProcesses::ProcessId pid)
{
    // sum the catches during the base period, after the harvest timestep
    if (pid == SystemProcesses::SP_Harvest) {
        for (int f=0; f < ::Fisheries.entries(); ++f) {
            for (int r=0; r < Geographer::nRegions(); ++r) {
                catches_(f, r, ::SystemClock->currentYearIndex(), ::SystemClock->currentTimeIndex()) 
                    += ::Fisheries[f].legalCatch(Geographer::region(r));
            }
        }
    }
}


// return a requested catch value
double TrackDetailedFisheryCatches::
basePeriodTotalCatch(const Fishery* fishery, int region, int time, int baseStartYearIndex,
                     int baseEndYearIndex) const
{
    double total=0;
    for (int year=baseStartYearIndex; year <= baseEndYearIndex; ++year) {
        total += catches_(fishery->index(), region, year, time);
    }
    return total;
}

double TrackDetailedFisheryCatches::
basePeriodAverageCatch(const Fishery* fishery, int region, int time, int baseStartYearIndex, 
                       int baseEndYearIndex) const
{
    double result = basePeriodTotalCatch(fishery, region, time, baseStartYearIndex, baseEndYearIndex);
    result /= (baseEndYearIndex - baseStartYearIndex) + 1;
    return result;
}

double TrackDetailedFisheryCatches::
totalCatch(const Fishery* fishery, const TimeStamp& first, const TimeStamp& last) const
{
    double total=0;
    for (TimeStamp time=first; time <= last; ++time) {
        for (int region=0; region < Geographer::nRegions(); ++region) {
            total += catches_(fishery->index(), region, time.yearIndex(), time.timeIndex());
        }
    }
    return total;
}

State* TrackDetailedFisheryCatches::
newSaveState()
{
    return new TrackDetailedFisheryCatchState(this); 
}
