/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

#include "pragma.h"

#include <iostream.h>
#include <stdlib.h>

#include "log.h"
#include "init.h"
#include "Simulation.h"
#include "DataRequestManager.h"
#include "FisheryManager.h"
#include "CohortState.h"
#include "IterationManager.h"

//#define DoQuantify
#ifdef DoQuantify
#include "pure.h"
#endif 

// common routine to perform core execution without any GUI.  does not
// include initializaton.  parameter is whether this is the first execution
// in a session (as in the first iteration of a calibration run).

static void
executeNonGui()
{
    LogMsg(L_Msg, "scenario run:\n");
    Simulation::run_scenario();
    DataRequestManager::outputAll();
    LogMsg(L_DbgMsg, "stats printed.\n");
    LogMsg(L_Msg, "simulation complete.\n");
}

#ifndef _AFXDLL // console app
main(int argc, char **argv)
{
#ifdef DoQuantify
    QuantifyStopRecordingData();
    QuantifyClearData();
#endif

    short retcode=0;

#ifndef DEBUG
    try {
#endif //DEBUG
        //    Args args(argc, argv);
        if (::ModelConfig() < 0){
            ::LogMsg(L_Fatal, "unable to initialize; exiting...\n");
            return(-1);
        }
#ifdef DoQuantify
//        QuantifyDisableRecordingData();
#endif // DoQuantify

// test multiple iterations
//        CohortState* cohortState = new CohortState;

        ::executeNonGui();
        
#ifndef DEBUG
    }
    catch (...){
        // fatal exception.  try to get one last message out to user.
        ::LogMsg(L_Force, "fatal error encountered.\n");
        retcode = -1;
    }
#endif // DEBUG

    ::SimulationWrapup();
/*
// test second execution
              FisheryManager::config();
              IterationManager::config();
              ::ModelDataRead("calib.data");
              ::ModelInit();
              cohortState->restoreState();
              ::executeNonGui();
              ::SimulationWrapup();
              delete cohortState;
*/
    ::ModelCleanup();
    return(retcode);
}
#else // activeX app

static CohortState* cohortState=0;

short activeXentry()
{
    short retcode=0;
    try {
        if (::ModelConfig() < 0) {
            ::LogMsg(L_Error, "unable to initialize; exiting...\n");
            // as an activeX library, close the logfile between simulation runs
            LogLevel::cleanup();
            return(-1);
        }
        cohortState = new CohortState;
        executeNonGui();
    }
    catch (...){
        // fatal exception.  try to get one last message out to user.
        ::LogMsg(L_Force, "fatal error encountered.\n");
        retcode = -1;
    }
    ::SimulationWrapup();
    // as an activeX library, close the logfile between simulation runs
    LogLevel::cleanup();
        
    return(retcode);
}

// routine for reentering a calibration.  assumes that the first calibration iteration
// has already occurred.  at this point the model is still configured, and most data is
// in place.  file "calib.data" will be read in to allow the user to selectively change 
// some items.  the original cohorts will be restored, and the policy table (FisheryManager)
// and IterationManager will be reset to defaults.  finally, a new simulation will be invoked.

short activeXcalibrationReentry()
{
    short retcode=0;
    try {
        LogLevel::config("coast_log.txt");

        // this is a little bogus, since it relies on specific knowledge about the nature
        // of the PSC configuration.  specifically, all special policies (e.g. quotas) are
        // created on the fly by PolicyControls, so it is ok to tell the FisheryManager to
        // reset the PolicyTable to default values.  similarly, all IterationControls are
        // created by PolicyControl objects, so it is safe (and necessary) to have the
        // IterationManager clear them.
        FisheryManager::config();
        IterationManager::config();

        // read new data, init, restore cohorts, and execute
        ::ModelDataRead("calib.data");
        ::ModelInit();
        cohortState->restoreState();
        ::executeNonGui();
    }
    catch (...){
        // fatal exception.  try to get one last message out to user.
        ::LogMsg(L_Force, "fatal error encountered.\n");
        retcode = -1;
    }
    ::SimulationWrapup();
    // as an activeX library, close the logfile between simulation runs
    LogLevel::cleanup();
    return(retcode);
}


void activeXexit()
{
    delete cohortState;
    ::ModelCleanup();
}

/*
//test from standard env
main()
{
#ifdef DoQuantify
    QuantifyStopRecordingData();
    QuantifyClearData();
    activeXentry();
    QuantifyStartRecordingData();
#endif

//    activeXentry();
    activeXcalibrationReentry();
}
*/

#endif //_AFXDLL

