/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// model initializations


#include <iostream.h>
#include <assert.h>
#include <string.h>
#include "NamedPtrVec.h" 
#include "globals.h"
#include "init.h"
#include "log.h"
#include "Stock.h"
#include "Fishery.h"
#include "FisheryUnit.h"
#include "FisheryManager.h"
#include "CohortManager.h"
#include "TopLevelParser.h"
#include "HarvestManager.h"
#include "DataRequestManager.h"
#include "StandardDataOutputManager.h"
#include "OutputFisheryCatches.h"
#include "OutputPSCStockEscapements.h"
#include "Historian.h"
#include "PolicyControlManager.h"
#include "IterationManager.h"
#include "MaturationManager.h"
#include "NaturalMortalityManager.h"
#include "MigrationManager.h"


/*
 * there are three phases to model startup: config, data read, and init.  config reads the 
 * configuration data file and allows managers to configure themselves, meaning
 * that dynamic memory is allocated and tables are dimensioned.  the data read phase
 * handles input of general model data.  init allows managers to perform final initializations,
 * such as setting default values.
 *
 * ModelConfig() executes all three phases.  in addition, the data read and init phases may
 * be invoked separately for subsequent model iterations.
 * subroutines typically handle the err messeges in this routine.
 *	return: 0  - success
 *		   -1  - error
 */

int ModelConfig()
{
    /* the following operations are order dependent:
     * read the setup data; allocate global structures; read remaining data
     */

    // first set up the logfile
    LogLevel::config("coast_log.txt");

    // GROT should be configurable filename
    try {
        TopLevelParser config("config.data");
        config.parse();
    } catch (ParseError& err) {
    	LogMsg(L_Error, "%s\n", (const char *) err.errorMessage());
        LogMsg(L_Error, "Configuration error.  Exiting.\n");
        return -1;
    }

    Geographer::checkConfig();
    Chronograph::checkConfig();

    // standard data ouput config must come before data request manager
    // GROT should be configurable output filename
    StandardDataOutputManager::config("coast_output.txt");
    DataRequestManager::configAll();

    // remaining data dependent inits, order dependent
    for (int i = 0; i < Stocks.num(); ++i)
    	Stocks[i].config();
    for (i=0; i < Fisheries.num(); ++i)
        Fisheries[i].config();

    FisheryManager::config();
    HarvestManager::config();
    CohortManager::config();	
    IterationManager::config();
    PolicyControlManager::config();
    MaturationManager::config();
    NaturalMortalityManager::config();

    int retcode = 0;
    // execute data read followed by init if successfull
    if ((retcode = ::ModelDataRead("coast.data"))
        || (retcode = ::ModelInit()))
        return retcode;
    else 
        return 0;
}

int ModelDataRead(const char* filename)
{
    // parser routines
    LogMsg(L_Msg, "Parsing data; this may take a while...\n");
    try {
        TopLevelParser data(filename);
        data.parse();
    } catch (ParseError& err) {
        LogMsg(L_Error, "%s", (const char *) err.errorMessage());
        // GROT some errors should probably be fatal here ?
    }
    LogMsg(L_DbgMsg, "ending parse.\n");
    return 0;
}

// initializations for each model run
int ModelInit()
{
    // create and schedule various DataRequests
    // (these filenames are hard coded)
    DataRequestManager::schedule(new OutputFisheryCatches("stndcat.prn"));
    DataRequestManager::schedule(new OutputPSCStockEscapements("stndesc.prn"));

    // these two Managers also schedule DataRequests
    Historian::init();
    StandardDataOutputManager::init();
    // initialized scheduled requests
    DataRequestManager::initAll();

    // some sanity checking on the configured model.  tables which are assumed 
    // to be filled with valid entries are checked here.  other Managers 
    // requiring initialization also execute at this time.
    MigrationManager::init();
    IterationManager::init();

    return 0;
}

// there are two phases of wrapup.  SimulationWrapup() occurs after every simulation
// run.  ModelCleanup() occurs only prior to final exit.
 
// standard wrapups to occur after each simulation
void SimulationWrapup()
{
    CohortManager::simulationWrapup();
    StandardDataOutputManager::simulationWrapup();
    DataRequestManager::simulationWrapup();
}

// final cleanup before exit - delete global objects and so forth
void ModelCleanup()
{
    delete ::SystemClock;
    ::SystemClock = 0;
    PolicyControlManager::cleanup();
    ::Fisheries.clearAndDestroy();
    ::Stocks.clearAndDestroy();
    LogLevel::cleanup();
}

