/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

#include "CNR_SeasonLengthMultipleEncounter.h"
#include "HarvestManager.h"
#include "Fishery.h"
#include "Region.h"
#include "log.h"
#include <math.h>

double CNR_SeasonLengthMultipleEncounter::
legal_ratio(const Fishery& f, const Region& r) const
{
    double rel_hr = f.relHr(r);
    
    if (rel_hr == 0) {
        LogMsg(L_Warning, "relative harvest rate = 0.0 in fishery %s,"
            "\n\tCNR computations will be inaccurate.",
            f.get_name());
        return 0;
    }

    return ((legalReleaseMortRate_ + f.dropOffMortRate(r)) * legal_sel * (cnr_season_len / legal_season_len));
}

// scalar for multiple encounters
double CNR_SeasonLengthMultipleEncounter::
cnrMultiEncScalar(const HvMort* hv, double selectivity) const
{
    double cnrScalar = 1.0;
#ifdef CNRS_FIXED
    double adjust = cnr_season_len / (cnr_season_len + legal_season_len);
    if (adjust > 1.0) {
        double baseHr = HarvestManager::baseHr(hv);
        double adjBaseHr = adjust * baseHr * selectivity;
        double relMortRate = legalReleaseMortRate_ + hv->fishery().dropOffMortRate(hv->region()); 
        if (adjBaseHr <= 0.0 || relMortRate == 0.0)
            cnrScalar = 0.0;
        else {
            double tempRate = -log(1.0 - adjBaseHr) * relMortRate * adjust;
            double exploitationRate = 1.0 - exp(-tempRate);
            cnrScalar = exploitationRate / (adjBaseHr * relMortRate);
        }
    }
#endif
    return cnrScalar;
}


double CNR_SeasonLengthMultipleEncounter::
sublegalMort(const HvMort* hv) const
{
    // parent class contains the standard mort equation, which uses the virtual
    // "ratio" fns.  here we also factor in the cnrScalar
    return (CNR_SeasonLength::sublegalMort(hv) * cnrMultiEncScalar(hv, sublegal_sel));
}


double CNR_SeasonLengthMultipleEncounter::
legalMort(const HvMort* hv) const
{
    // parent class contains the standard mort equation, which uses the virtual
    // "ratio" fns.  here we also factor in the cnrScalar
    return (CNR_SeasonLength::legalMort(hv) * cnrMultiEncScalar(hv, legal_sel));
}
