/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "VulnerabilityTableParser.h"
#include "GenericArrayFactory.h"
#include "GenericDefaultArray.h"
#include "ShakerTokens.h"


// ctor utility fns
void
VulnerabilityTableParser::initKeymap()
{
    keymap_.insert(ShakerTokens::VULNERABLE, vulnerable);
    keymap_.insert(ShakerTokens::INVULNERABLE, invulnerable);
    initGenArrayKeymap(keymap_);
}

void
VulnerabilityTableParser::initState()
{
    // configure the initial state with the correct id list.
    State* state = new State;
    state->context = "ShakerVulnerabilityData";
    configTable(state);
    stateStack.push(state);
}

VulnerabilityTableParser::VulnerabilityTableParser(Parser& p, RWCString dimension)
    : GenArrayParser(p), dimension_(dimension), 
      vulnerabilityTable_(auto_ptr<GenericArray<bool> >(new GenericDefaultArray<bool>(false)))
{
    initState();
    initKeymap();
}

// create a generic array for vulnerability data of the requested
// dimension size.  also add the list of generic ids to index every
// element of the array to the current state.

void
VulnerabilityTableParser::configTable(State* state)
{
    GenericArrayFactory<bool> vFactory;
    GenericArrayFactory<bool>::GenericArrayType arraytype = vFactory.genericArrayType(dimension_); 
    if (arraytype != vFactory.NTypes) {
        // this is slightly complicated.  the factory makes a new genarray, handed to a local 
        // VulnerabilityTable via auto_ptr.  the local (compiler temporary) vtab is used
        // to initialize the reference counted vtab kept by this class.  the class member 
        // one is later "copied" to the outside world.  when the last vtab disappears, 
        // all heap resident items are automagically deleted.
        vulnerabilityTable_ = 
            VulnerabilityTable(auto_ptr<GenericArray<bool> > (vFactory.newGenericArrayFn1(arraytype, false)));
        vulnerabilityTable_.newGenericIndexList(state->ids); // create the proper list of generic ids
    }
    else {
        // unidentified dimension.  log an error, but allow this
        // parser to continue with an empty id list in the state, so
        // as to skip as much data as possible without generating
        // further messages.
        logError(pError(ParseError::BadData,
            dimension_ + " is not a valid configuration for"
            + state->context + "; discarding data")
            .errorMessage());
    }
}


// mark "true" at every generic id region in the current list
void
VulnerabilityTableParser::vulnerable()
{
    // iterate through the ids in the current state
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        vulnerabilityTable_[*id] = true;
        ++iter;
    }
    // pop one level of state after processing the data
    parseEnd();
}

// mark "false" at every generic id region in the current list
void
VulnerabilityTableParser::invulnerable()
{
    // iterate through the ids in the current state
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        vulnerabilityTable_[*id] = false;
        ++iter;
    }
    // pop one level of state after processing the data
    parseEnd();
}

